
import trajectory
coordinate_dict = {'ascii' : trajectory.CoordinateParser,
                   'netcdf': trajectory.NetCDFCoordinateReader}

velocity_dict   = {'ascii' : trajectory.VelocityParser,
                   'netcdf': trajectory.NetCDFVelocityReader}

restart_dict    = {'restart' : trajectory.RestartParser}

import topology
topology_dict  = {'amber'  : topology.TopologyParser}

converter_dict = {}
available_pottypes = ['amber-base', 'amber',  'amber94', 'amber96', 'amber99',
                      'amber99SB', 'amber03', 'amber10', 'amber12SB' ]
converter_dict = { ('amber', pot_type):topology.Format2AmberBaseConverter
                   for pot_type in available_pottypes }

coordinate_writer_dict = {
        'ascii' : trajectory.AsciiCoordinateWriter,
        'netcdf': trajectory.NetCDFCoordinateWriter}

velocity_writer_dict = {
        'ascii' : trajectory.AsciiVelocityWriter,
        'netcdf': trajectory.NetCDFVelocityWriter}



"""
Gives functions to create topology and trajectory variables from files indicated in .cfg configuration file.
"""

import os, sys
import traceback
import itertools as it
topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)

################################################################################
from exception import WurpException
class FormatNotFoundError(WurpException): pass
class ParserNotFoundError(WurpException): pass
class ConverterNotFoundError(WurpException): pass
class NotFoundDictionary(WurpException): pass
class StepNumberInvalidError(WurpException): pass

import trans_rot

################################################################################
def get_tplprm(file_format, format_setting, potential_type, use_atomtype):

    # decide topology parser
    TopologyParser = get_topology_parser(file_format)
    tpl_parser = TopologyParser( format_setting.topology_file[0] )
    tpl_parser.parse()

    # convert topology to forcefield
    TopologyConverter = get_topology_converter(file_format, potential_type)
    tpl_conv = TopologyConverter(tpl_parser, use_atomtype)
    tpl_conv.convert(format_setting)

    return tpl_conv

def get_tplprm_simple(file_format, fp, use_atomtype=False):

    # decide topology parser
    TopologyParser = get_topology_parser(file_format)
    tpl_parser = TopologyParser( fp )
    tpl_parser.parse()

    # convert topology to forcefield
    TopologyConverter = get_topology_converter(file_format, file_format)
    tpl_conv = TopologyConverter(tpl_parser, use_atomtype)
    tpl_conv.convert()

    return tpl_conv


def gen_phase_trajectory(file_format, fmt_section,
                         first_last_interval, natom, use_pbc,
                         rem_trans, rem_rotate, target_atoms, masses,
                         logger):

    # define the parser class
    if fmt_section.target == 'restart':

        rst_format = fmt_section.restart_format
        rst_filname = fmt_section.restart_file[0]

        RestartParser = get_restart_parser(rst_format)

        rst_parser = RestartParser(rst_filname, natom)
        crd, vel, pdc = rst_parser.parse()

        # remove translation and rotation
        new_crd, new_vel = trans_rot.get_crd_vel_trans_rot_removed(
                masses, crd, vel, target_atoms, rem_trans, rem_rotate)
        crd_pbcs = [(0, new_crd, None)]
        vel_pbcs = [(0, new_vel, None)]

    elif fmt_section.target == 'trajectory':

        crd_format = fmt_section.coordinate_format
        vel_format = fmt_section.velocity_format
        crd_filename = fmt_section.coordinate_file
        vel_filename = fmt_section.velocity_file

        CoordinateParser = get_coordinate_parser(crd_format)
        VelocityParser   = get_velocity_parser(vel_format)

        crd_pbcs = gen_trajectory(fmt_section.coordinate_file,
                CoordinateParser, natom, logger, use_pbc, 
                first_last_interval=first_last_interval)

        vel_pbcs = gen_trajectory(fmt_section.velocity_file,
                VelocityParser, natom, logger, False,
                first_last_interval=first_last_interval)

    else:
        pass

    first, last, inter = first_last_interval
    logger.info_title('The range of trajectories')
    logger.info('first, last, interval of step = {} {} {}'.format(
        first, last, inter))
    yield

    istep = 0
    for (crd_data, vel_data) in it.izip(crd_pbcs, vel_pbcs):
        cstep_crd, crd, pbc_crd = crd_data
        cstep_vel, vel, pbc_vel = vel_data
        istep += 1
        logger.set_curstep(istep)

        if cstep_crd != cstep_vel:
            msg = 'crd step = {}, but vel step = {}.'
            raise StepNumberInvalidError( msg.format(cstep_crd, cstep_vel) )

        logger.debug_cycle('\n\n' + 20*'=' + 'STEP {}'.format(istep) + 20*'=')
        logger.debug_cycle('    current step = {}'.format(cstep_crd))
        logger.info_cycle('*** ISTEP = {}, CURRENT STEP = {} ***'
                .format(istep, cstep_crd))
        logger.debug_cycle()

        # remove translation and rotation
        new_crd, new_vel = trans_rot.get_crd_vel_trans_rot_removed(
                masses, crd, vel, target_atoms, rem_trans, rem_rotate)

        yield cstep_crd, (new_crd, new_vel, pbc_crd)

def gen_matrix_ensemble(file_format, fmt_section, first_last_interval, natom):
    # define the parser class

    return istep, matrix

################################################################################

def get_topology_parser(format):
    return get_parser(format, 'topology')

def get_coordinate_parser(format):
    return get_parser(format, 'coordinate')

def get_velocity_parser(format):
    return get_parser(format, 'velocity')

def get_restart_parser(format):
    return get_parser(format, 'restart')

def get_topology_converter(format, forcefield):
    dictname = 'converter_dict'
    key = (format, forcefield)

    # get the parser's directory
    parser_path = os.path.dirname(__file__)

    # get the plugin names
    modpaths = []
    for fd in os.listdir(parser_path):
        abspath = os.path.join(parser_path, fd)
        if os.path.isdir(abspath) and not fd.startswith('_'):
            modpaths.append(abspath)

    for mpath in modpaths:
        modname = os.path.split(mpath)[-1]
        try:
            module = load_module(modname, parser_path)
            if hasattr(module, dictname):
                dictionary = getattr(module, dictname)
            else:
                raise NotFoundDictionary(dictname)

            if key in dictionary:
                Converter = dictionary[key]
                break

        except ImportError:
            print(traceback.format_exc())
            # raise FormatNotFoundError(format)

        except:
            raise

    else:
        raise ParserNotFoundError(key)

    return Converter


def gen_trajectory(traj_files, Parser, natom, logger=None,
        use_pbc=False, first_last_interval=(1, -1, 1)):
    """Generate trajectory from files."""

    # logger
    class Logger:
        def info(self, *args, **kwds):
            pass

    if logger is None:
        logger = Logger()

    # range_line
    first, last, inter = first_last_interval

    int_count = 0
    istep_all = 0
    last = 10**18 if last <= -1 else last

    # do loop the trajectory files
    for traj_fn in traj_files:
        logger.info('*** reading trajectory file ***')
        logger.info('    {}'.format(traj_fn) )
        logger.info()
        parser = Parser(traj_fn, natom, use_pbc=use_pbc)
        # do loop the trajectory in a file
        for istep_1, traj in enumerate(parser):
            int_count += 1
            istep = istep_1 + 1
            istep_all += 1
            body, pbc = traj

            if istep_all < first:
                pass
            elif istep_all == first:
                yield istep_all, body, pbc
                int_count = 0
            elif (first < istep_all <= last) and (int_count == inter):
                int_count = 0
                yield istep_all, body, pbc
            elif last < istep_all:
                break
            else:
                continue

        parser.close()

def gen_matrix(matrix_files, Parser, natom, logger):
    pass


################################################################################
def get_parser(format, parser_name):
    dictname = parser_name + '_dict'

    # get the parser's directory
    parser_path = os.path.dirname(__file__)

    # get the plugin names
    modpaths = []
    for fd in os.listdir(parser_path):
        abspath = os.path.join(parser_path, fd)
        if os.path.isdir(abspath) and not fd.startswith('_'):
            modpaths.append(abspath)

    for mpath in modpaths:
        modname = os.path.split(mpath)[-1]
        try:

            # get parser dictionary
            module = load_module(modname, parser_path)
            if hasattr(module, dictname):
                dictionary = getattr(module, dictname)
            else:
                raise NotFoundDictionary(dictname)

            # get Parser
            if format in dictionary:
                Parser = dictionary[format]
                break

        except ImportError:
            print(traceback.format_exc())
            # raise FormatNotFoundError(format)

        except:
            raise

    else:
        raise ParserNotFoundError(format)

    return Parser


import imp
def load_module(modname,  basepath):
    f,n,d = imp.find_module(modname, [basepath])
    return imp.load_module(modname, f, n, d)


if __name__ == '__main__':
    TopologyParser = get_topology_parser('presto')
    tparser = TopologyParser('./presto_v2/test/ala_all.tpl')
    tparser.parse()

    Converter = get_topology_converter('presto', 'amber99')
    con = Converter(tparser)
    con.convert()



from __future__ import print_function

# standard modules
import os, sys

# wurp modules
topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path: sys.path.insert(0, topdir)
from exception import WurpException

# exceptions
class TrajectoryExistsError(WurpException): pass


class Writer:

    def __init__(self, dynamics_setting):

        dyn_sets = dynamics_setting

        crd_freq = dyn_sets.crds_frequency
        vel_freq = dyn_sets.vels_frequency
        last = 10**18

        crds_range = (crd_freq, last, crd_freq)
        vels_range = (crd_freq, last, vel_freq)

        import parser.writer as pw

        # coordinate
        if dyn_sets.crds_frequency >= 1:
            self.__crds_writer = pw.Writer(
                    trj_fp      = dyn_sets.crds_file[0] , 
                    trj_fmt     = dyn_sets.trj_format   , 
                    dt          = dyn_sets.dt           , 
                    is_vel      = False                 , 
                    fst_lst_int = crds_range)
        else:
            self.__crds_writer = None

        # velocity
        if dyn_sets.crds_frequency >= 1:
            self.__vels_writer = pw.Writer(
                    trj_fp      = dyn_sets.vels_file[0] , 
                    trj_fmt     = dyn_sets.trj_format   , 
                    dt          = dyn_sets.dt           , 
                    is_vel      = True                  , 
                    fst_lst_int = vels_range)

        else:
            self.__vels_writer = None

    def write_header(self):

        if self.__crds_writer:
            self.__crds_writer.write_header()

        if self.__vels_writer:
            self.__vels_writer.write_header()

    def write(self, istp, crd=None, vel=None):
        """Write trajectory."""

        if self.__crds_writer:
            self.__crds_writer.write(istp, crd)

            # The box is not implemented in dynamics run.
            #self.__crds_writer.write(istp, crd, box)

        if self.__vels_writer:
            self.__vels_writer.write(istp, vel)




"""Tool console scripts."""

from __future__ import print_function

# Standard module
import argparse
import wurp.tool as tool
from wurp.console import exec_command


manual = {
          'tool': 'Scripts to process irEC data',
          'renum-residue': 'Renumbering the residue numbers'
          }


def arg_tool(parser=None):
    # Initialize
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['tool']
    parser.prog = 'tool'

    # Add commands from manual dictionnary as subparsers
    sp = parser.add_subparsers(dest='command', help='Wurp tool commands')

    cmd_list = manual.keys()
    cmd_list.remove('tool')
    cmd_sp = {cmd: sp.add_parser(cmd, help=manual[cmd]) for cmd in cmd_list}

    # Add deeper subparsers
    from wurp.tool.sele.console import manual as sele_manual
    cmd_sp['sele'] = sp.add_parser('sele',
                                   help=sele_manual['sele'])

    # Configure subparsers
    arg_renum_res(cmd_sp['renum-residue'])

    from wurp.tool.sele.console import arg_sele
    arg_sele(cmd_sp['sele'])


def arg_renum_res(parser=None):
    return_parser = parser is None
    if return_parser:
        parser = argparse.ArgumentParser()

    parser.description = manual['renum-residue']
    parser.prog = 'renum-residue'
    parser.set_defaults(func=tool.renum_residue.main)

    # Check if there is no input
    # from sys import stdin
    # if stdin.isatty():
    #     print('no stdin')
    #     parser.add_argument('ec_fn',
    #                         help=('Interresidue energy conductivity '
    #                               '(irEC) filename.'))
    # else:
    #     parser.set_defaults(ec_file=stdin)

    parser.add_argument('renums', nargs='+',
                        help=('Residue to renumber and which number to assign.'
                              'Ex: 2:10 would add 9 to every residue numbers, '
                              'starting at residue number 2.'))
    parser.add_argument('-f', '--ec-fn', dest='ec_fn', default=None,
                        help=('[Optional] irEC data filename. If not '
                              'provided requires standard input.'))

    if return_parser:
        return parser


def main():
    parser = arg_tool()
    exec_command(parser)


if __name__ == '__main__':
    main()



#! /usr/bin/env python
from __future__ import print_function

import numpy

from wurp import get_tpl
from wurp.script.conv_trj import gen_trj
from wurp.script.analyze.lib_pickup import pickup
from wurp.table.group import gen_residue_group

def pickup_respairs(trj_fns, input_trj_fmt, prmtop_fn, prmtop_fmt, interval=1,
         cutoff_method='nearest', cutoff=5., trim_resnames=[],
         format='{rid:05}_{rname}', is_union=True, ext_resids='',
         is_both=False, **kwds):
    """Pick up the residue pair table to given file as a ndx format."""

    # Get parameters
    cutoff = float(cutoff)
    cutoff2 = cutoff*cutoff

    # Get topology
    tpl = get_tpl(prmtop_fmt, prmtop_fn)
    natom = tpl.get_natom()

    # Generate residue group
    res_info  = tpl.get_residue_info()
    atom_info = tpl.get_atom_info()
    rname_iatoms_pairs = list( (rname, list(numpy.array(iatoms)))
            for rname, iatoms in gen_residue_group(
                res_info, atom_info, name_fmt=format))

    resnames = [ resname for resname, iatoms in rname_iatoms_pairs ]
    nres = len(resnames)

    # Get trajectory
    crd_parser = gen_trj(tpl, trj_fns, input_trj_fmt,
            trj_type='crd', input_fst_lst_int=[(1, -1, interval)],
            use_pbc=False)
    crds = ( crd for itraj, crd, box in crd_parser )

    res_beg_end_pairs = [ (min(iatoms), max(iatoms))
            for rname, iatoms in rname_iatoms_pairs ]

    pickup.res_beg_end_pairs = numpy.array(res_beg_end_pairs)
    pickup.cutoff2 = cutoff2
    if cutoff_method == 'com':
        pickup.cutoff_method = 1
    elif cutoff_method == 'nearest':
        pickup.cutoff_method = 2
    elif cutoff_method == 'farthest':
        pickup.cutoff_method = 3
    else:
        pass

    respair_table_traj = (gen_respair_table_fortran(crd, nres, resnames)
            for crd in crds )

    respair_iter = gen_respair_table_over_traj(
            respair_table_traj, is_union)

    # setup excluded resids
    def gen_excluded_respair(respairs, excluded_resids):
        ext_resids = []
        for ext_resid in excluded_resids:
            rid_beg, rid_end = ext_resid.split(':')
            rid_beg, rid_end = int(rid_beg), int(rid_end)
            ext_resids.extend( range(rid_beg, rid_end+1) )

        # print(ext_resids)

        flag_i = False
        for rname_i, rnames_j in respairs:
            rid_i = int(rname_i.split('_')[0])
            if rid_i in ext_resids: flag_i = True

            new_rnames_j = []

            for rname_j in rnames_j:
                rid_j = int(rname_j.split('_')[0])
                if flag_i and rid_j in ext_resids: continue

                new_rnames_j.append(rname_j)

            else:
                flag_i = False

            if new_rnames_j:
                yield rname_i, new_rnames_j

    def gen_excluded_respair_both(respairs, excluded_resids):
        ext_resids = []
        for ext_resid in excluded_resids:
            rid_beg, rid_end = ext_resid.split(':')
            rid_beg, rid_end = int(rid_beg), int(rid_end)
            ext_resids.extend( range(rid_beg, rid_end+1) )

        # print(ext_resids)

        for rname_i, rnames_j in respairs:
            rid_i = int(rname_i.split('_')[0])
            if rid_i in ext_resids: continue

            new_rnames_j = []

            for rname_j in rnames_j:
                rid_j = int(rname_j.split('_')[0])
                if rid_j in ext_resids: continue

                new_rnames_j.append(rname_j)

            if new_rnames_j:
                yield rname_i, new_rnames_j

    # write residue pair data
    print('# residue pair within {:4.2f}'.format(cutoff))
    npair = 0
    if ext_resids and not is_both:

        for rname_i, rnames_j in gen_excluded_respair(
                respair_iter, ext_resids):

            if rnames_j:
                npair += len(rnames_j)
                print('[{}]'.format(rname_i))
                print('  '.join(rnames_j))

    elif ext_resids and is_both:

        for rname_i, rnames_j in gen_excluded_respair_both(
                respair_iter, ext_resids):

            if rnames_j:
                npair += len(rnames_j)
                print('[{}]'.format(rname_i))
                print('  '.join(rnames_j))

    else:
        for rname_i, rnames_j in respair_iter:

            if rnames_j:
                npair += len(rnames_j)
                print('[{}]'.format(rname_i))
                print('  '.join(rnames_j))


    print('# number of pairs = ', npair)


def cal_dist2(p1, p2):
    # return sum( (p1 - p2)**2 )
    return (p1[0]-p2[0])**2 + (p1[1]-p2[1])**2 + (p1[2]-p2[2])**2

def zip_double(data_iter):

    for i, e1 in enumerate(data_iter):
        for j, e2 in enumerate(data_iter):
            if j <= i: continue
            yield e1, e2

def gen_respair_table_fortran(crd, nres, resnames):

    cands = pickup.get_candidate_pairs(crd, nres)

    for ires_1 in range(nres-1):
        jreslist = []
        for jres_1 in range(ires_1+1, nres):
            if cands[ires_1, jres_1]: jreslist.append( resnames[jres_1] )

        yield resnames[ires_1], jreslist

def gen_respair_table(crd, resnames, rname_iatoms_pairs, cutoff_method,cutoff2):

    res_crds = [ crd[iatoms] for rname, iatoms in rname_iatoms_pairs ]

    for i, (rname_i, rcrd_i) in enumerate(zip(resnames, res_crds)):
        resj_cands = []
        for j, (rname_j, rcrd_j) in enumerate(zip(resnames, res_crds)):
            if j <= i: continue

            if cutoff_method(rcrd_i, rcrd_j, cutoff2):
                resj_cands.append(rname_j)

        yield rname_i, resj_cands


def is_com(cutoff2):

    def _judge(crd_i, crd_j):
        com_i = numpy.sum(crd_i, 0)
        com_j = numpy.sum(crd_j, 0)
        return cal_dist2(com_i, com_j) <= cutoff2

    return _judge

def is_nearest(cutoff2):

    def _judge(crd_i, crd_j):
        for r_i in crd_i:
            for r_j in crd_j:
                if cal_dist2(r_i, r_j) <= cutoff2:
                    return True

        else:
            return False

    return _judge

def is_farthest(cutoff2):

    def _judge(crd_i, crd_j):
        for r_i in crd_i:
            for r_j in crd_j:
                if cal_dist2(r_i, r_j) >= cutoff2:
                    return False

        else:
            return True

    return _judge

def gen_respair_table_over_traj(respair_table_traj, is_union=True):

    table = {}
    rnames = []

    for itrj, respair_table in enumerate(respair_table_traj,1):
        print('# itraj = ', itrj)

        for rname_i, rnames_j in respair_table:

            if rname_i not in table:
                table[rname_i] = set(rnames_j)
                rnames.append( rname_i )
                continue

            if is_union:
                table[rname_i] = table[rname_i] | set(rnames_j)
            else:
                table[rname_i] = table[rname_i] & set(rnames_j)

    for rname_i in rnames:
        yield rname_i, list(table[rname_i])



if __name__ == '__main__':
    from wurp.console import arg_respairs, exec_command

    parser = arg_respairs()
    exec_command(parser)



from .conv_trj import do_convert_only, gen_trj
from .distance import do_distance
from .adjustvel import adjust_vel
from .cal_tc import cal_tc
from .mask import do_mask
from .graph_een import graph_een
from .sum_acf import sum_acf

from . import analyze



from __future__ import print_function

import os, sys
import math
import numpy
from math import sqrt, pi
from abc import abstractmethod, abstractproperty, ABCMeta

topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)
import utility

wurp_dir = os.environ['CURP_HOME']
script_dir = os.path.abspath(os.path.join(wurp_dir, 'script'))
if script_dir not in sys.path:
    sys.path.insert(0, script_dir)
import pdb_ as PDB

# wurp modules
import clog as logger
from exception import WurpException

class NotFoundPropertyError(WurpException): pass
class NumberOfTargetError(WurpException): pass
class VolumeNotDefinedError(WurpException): pass
class TestError(WurpException): pass
class NumberOfGroupError(WurpException): pass

def nproperty(function):
    """Decorator to easy use property.

    Usage:

    class A(object):
        def __init__(self):
            self.__x = None

        @nproperty
        def x():

            test_val_list = [] # use in get and set

            def get(self):
                print('Get property x.')
                return self.__x

            def set(self, x):
                print('Set property x: ', x, '.')
                self.__x = x

            def del_(self, x):
                del self.__x

    a = A()
    a.x = 5
    print a.x
    => 5
    getattr(a, 'x') => 5
    setattr(a, 'x', 9)
    print a.x
    => 9
    """

    func_locals = {'doc':function.__doc__}
    def probe_func(frame, event, arg):
        if event == 'return':
            locals = frame.f_locals

            prop_dict = {}
            for name, method in locals.items():
                if name.startswith('get'):
                    prop_dict['fget'] = method
                    get_method = method
                elif name.startswith('set'):
                    prop_dict['fset'] = method
                elif name.startswith('del'):
                    prop_dict['fdel'] = method
                # elif name.startswith('doc'):
                #     prop_dict['doc']  = method
                else:
                    pass

            func_locals.update( prop_dict )
            sys.settrace(None)
        return probe_func
    sys.settrace(probe_func)
    function()
    return property(**func_locals)

class UndefinedPropertyError(WurpException): pass
class VolumeSetting:

    def __init__(self, topology=None, setting=None,
            traj_parser=None, target_atoms=None, gname_iatoms_pairs=None):
        self.__tpl = topology
        self.__setting = setting
        self.__traj_parser = traj_parser
        self.__target_atoms = target_atoms
        self.__gname_iatoms_pairs = gname_iatoms_pairs

        self.__props = dict(
                natom               = 0  , 
                names               = [] , 
                elems               = [] , 
                resnames            = [] , 
                gname_iatoms_pairs  = [] , 
                target_atoms        = [] , 
                output_volume_file  = '' , 
                output_gvolume_file = '' , 

                vdw_radii = [] , 

                smve_rmax      = 2.5  , 
                smve_dr        = 0.01 , 
                smve_interval  = 1    , 
                smve_increment = 5    , 

                atomic_trajectory_file = '' , 
                group_trajectory_file  = '' , 

                voronoi_cutoff       = 6.0   , 
                voronoi_no_hydrogen  = False , 
                voronoi_solvation    = 'none' , 
                voronoi_probe_length = 2.4   , 
                voronoi_output_solvation_file = '',

                group_method = 'none'
        )


    def __str__(self):
        return str(self.__props)

    @nproperty
    def traj_parser():

        def get(self):
            return self.__traj_parser

        def set(self, value):
            self.__traj_parser = value

    @nproperty
    def natom():

        def get(self):
            if self.__tpl:
                return self.__tpl.get_natom()
            else:
                return self.__props['natom']

        def set(self, value):
            if self.__tpl is None:
                self.__props['natom'] = value

    @nproperty
    def names():

        def get(self):
            if self.__tpl:
                return self.__tpl.get_atom_info()['names']
            else:
                return self.__names

        def set(self, value):
            if self.__tpl is None:
                self.__props['names'] = value

    @nproperty
    def elems():

        def get(self):
            if self.__tpl:
                return self.__tpl.get_atom_info()['elems']
            else:
                return self.__elems

        def set(self, value):
            if self.__tpl is None:
                self.__props['elems'] = value

    @nproperty
    def resnames():

        def get(self):
            if self.__tpl:
                return self.__tpl.get_residue_info()['names']
            else:
                return self.__resnames

        def set(self, value):
            if self.__tpl is None:
                self.__props['resnames'] = value

    @nproperty
    def rids():

        def get(self):
            if self.__tpl:
                return self.__tpl.get_residue_info()['ids']
            else:
                return self.__rids

        def set(self, value):
            if self.__tpl is None:
                self.__props['rids'] = value

    @nproperty
    def target_atoms():

        def get(self):
            if self.__target_atoms:
                return self.__target_atoms
            else:
                return self.__props['target_atoms']

        def set(self, value):
            if self.__target_atoms is None:
                self.__props['target_atoms'] = value

    @nproperty
    def gname_iatoms_pairs():

        def get(self):
            if self.__gname_iatoms_pairs:
                return self.__gname_iatoms_pairs
            else:
                return self.__props['gname_iatoms_pairs']

        def set(self, value):
            if self.__gname_iatoms_pairs is None:
                self.__props['gname_iatoms_pairs'] = value

    @nproperty
    def output_volume_file():

        def get(self):
            if self.__setting:
                return self.__setting.volume.output_volume_file[0]
            else:
                return self.__props['output_volume_file']

        def set(self, value):
            if self.__setting is None:
                self.__props['output_volume_file'] = value

    @nproperty
    def output_gvolume_file():

        def get(self):
            if self.__setting:
                return self.__setting.volume.output_gvolume_file[0]
            else:
                return self.__props['output_gvolume_file']

        def set(self, value):
            if self.__setting is None:
                self.__props['output_gvolume_file'] = value

    @nproperty
    def vdw_radii():

        def get(self):
            if self.__tpl:
                return self.__tpl.get_atom_info()['vdw_radii']
            else:
                return self.__vdw_radii

        def set(self, value):
            if self.__tpl is None:
                self.__props['vdw_radii'] = value

    @nproperty
    def smve_rmax():

        def get(self):
            if self.__setting:
                return self.__setting.volume.smve_rmax
            else:
                return self.__props['smve_rmax']

        def set(self, value):
            if self.__setting is None:
                self.__props['smve_rmax'] = value

    @nproperty
    def smve_dr():

        def get(self):
            if self.__setting:
                return self.__setting.volume.smve_dr
            else:
                return self.__props['smve_dr']

        def set(self, value):
            if self.__setting is None:
                self.__props['smve_dr'] = value

    @nproperty
    def smve_interval():

        def get(self):
            if self.__setting:
                return self.__setting.volume.smve_interval
            else:
                return self.__props['smve_interval']

        def set(self, value):
            if self.__setting is None:
                self.__props['smve_interval'] = value

    @nproperty
    def smve_increment():

        def get(self):
            if self.__setting:
                return self.__setting.volume.smve_increment
            else:
                return self.__props['smve_increment']

        def set(self, value):
            if self.__setting is None:
                self.__props['smve_increment'] = value

    @nproperty
    def atomic_trajectory_file():

        def get(self):
            if self.__setting:
                return self.__setting.volume.atomic_trajectory_file[0]
            else:
                return self.__props['atomic_trajectory_file']

        def set(self, value):
            if self.__setting is None:
                self.__props['atomic_trajectory_file'] = value

    @nproperty
    def group_trajectory_file():

        def get(self):
            if self.__setting:
                return self.__setting.volume.group_trajectory_file[0]
            else:
                return self.__props['group_trajectory_file']

        def set(self, value):
            if self.__setting is None:
                self.__props['group_trajectory_file'] = value

    @nproperty
    def voronoi_no_hydrogen():

        def get(self):
            if self.__setting:
                return self.__setting.volume.voronoi_no_hydrogen
            else:
                return self.__props['voronoi_no_hydrogen']

        def set(self, value):
            if self.__setting is None:
                self.__props['voronoi_no_hydrogen'] = value

    @nproperty
    def voronoi_cutoff():

        def get(self):
            if self.__setting:
                return self.__setting.volume.voronoi_cutoff
            else:
                return self.__props['voronoi_cutoff']

        def set(self, value):
            if self.__setting is None:
                self.__props['voronoi_cutoff'] = value

    @nproperty
    def voronoi_solvation():

        def get(self):
            if self.__setting:
                return self.__setting.volume.voronoi_solvation
            else:
                return self.__props['voronoi_solvation']

        def set(self, value):
            if self.__setting is None:
                self.__props['voronoi_solvation'] = value

    @nproperty
    def voronoi_probe_length():

        def get(self):
            if self.__setting:
                return self.__setting.volume.voronoi_probe_length
            else:
                return self.__props['voronoi_probe_length']

        def set(self, value):
            if self.__setting is None:
                self.__props['voronoi_probe_length'] = value

    @nproperty
    def voronoi_output_solvation_file():

        def get(self):
            if self.__setting:
                return self.__setting.volume.voronoi_output_solvation_file[0]
            else:
                return self.__props['voronoi_output_solvation_file']

        def set(self, value):
            if self.__setting is None:
                self.__props['voronoi_output_solvation_file'] = value

    @nproperty
    def group_method():

        def get(self):
            if self.__setting:
                return self.__setting.wurp.group_method
            else:
                return self.__props['group_method']

        def set(self, value):
            if self.__setting is None:
                self.__props['group_method'] = value

class VolumeCalculatorBase:

    __metaclass__ = ABCMeta

    def __init__(self, volume_setting):
        self._setting = volume_setting
        self.__iatm_to_itars = None
        self.__do_write_volume = False
        self.__do_write_gvolume = False

        # prepare to write out volumes and group volumes data
        self.__count = 0
        if self._setting.output_volume_file != '':
            self.__do_write_volume = True

        if self._setting.output_gvolume_file != '':
            self.__do_write_gvolume = True

        self.__gnames = [ gname for gname, iatoms
                in self._setting.gname_iatoms_pairs ]

        self.prepare()

    @abstractmethod
    def prepare(self):
        return

    @abstractmethod
    def cal_volume(self, crd):
        return

    def get_volume(self, crd=[]):
        self.__count += 1
        self.__volumes = self.cal_volume(crd)

        # write volumes
        self.write_atomic_volume(self.__volumes)

        return self.__volumes

    def get_gvolume(self, crd=[]):
        """Sum up the volume in terms with given group and get its volume."""

        if self.__volumes is None:
            return None

        else:
            gvolumes = []
            iatm_to_itars = self.get_iatm_to_itars()

            for igrp_1, (gname, atoms) in enumerate(
                    self._setting.gname_iatoms_pairs):

                gvol = 0.0
                
                for iatm in atoms:
                    itar = iatm_to_itars[iatm-1]
                    if itar == 0: continue

                    gvol += self.__volumes[itar-1]

                gvolumes.append( gvol )

            # write volumes
            self.write_group_volume(gvolumes)

            return gvolumes

    def get_iatm_to_itars(self):
        if self.__iatm_to_itars is None:
            iatm_to_itars = numpy.zeros( [self._setting.natom], numpy.int)

            for itar_1, iatm in enumerate(self._setting.target_atoms):
                iatm_to_itars[iatm-1] = itar_1 + 1

            self.__iatm_to_itars = iatm_to_itars

        return self.__iatm_to_itars

    def get_itar_by_iatm(self, iatm=0):
        iatm_to_itars = self.get_iatm_to_itars()
        natom = len(iatm_to_itars)
        if iatm <= 0:
            return 0
        elif iatm > natom:
            return 0
        else:
            return iatm_to_itars[iatm-1]

    def write_atomic_volume(self, volumes):
        if not self.__do_write_volume: return

        filename = self._setting.output_volume_file
        names = self._setting.names

        fmt = '{id:>5d} {name:<10s}   {volume:>12.5f}'
        lines = []

        # for system
        file = open(filename, 'a')
        file.write('#BEGIN step ' + str(self.__count) + '\n')

        for iatm, volume in zip(self._setting.target_atoms, volumes):
            name = names[iatm-1]
            line = fmt.format(id=iatm, name=name, volume=volume)
            file.write(line + '\n')

        file.write('#END\n')
        file.close()

    def write_group_volume(self, gvolumes):
        if not self.__do_write_gvolume: return

        filename = self._setting.output_gvolume_file
        gnames = self.__gnames

        fmt = '{id:>5d} {name:<10s}   {volume:>12.5f}'
        lines = []

        # for system
        file = open(filename, 'a')
        file.write('#BEGIN step ' + str(self.__count) + '\n')
        logger.debug(gnames)
        logger.debug(gvolumes)

        for i_1, (name, volume) in enumerate( zip(gnames, gvolumes) ):
            line = fmt.format(id=i_1+1, name=name, volume=volume)
            file.write(line + '\n')

        file.write('#END\n')
        file.close()


class VDWVolumeCalculator(VolumeCalculatorBase):

    def __init__(self, volume_setting):
        self.__volumes = None
        VolumeCalculatorBase.__init__(self, volume_setting)

    def prepare(self):
        self.__target_atoms = self._setting.target_atoms
        self.__vdw_radii = self._setting.vdw_radii
        self.__volumes = self.cal_volume()

    def cal_volume(self, crd=[]):
        """Calculate the atom volumes from atom radii."""

        if self.__volumes is None:
            volumes = []
            for iatm in self.__target_atoms:
                radius = self.__vdw_radii[iatm-1]
                volumes.append( 4*pi/3 * radius**3 )
            self.__volumes = numpy.array(volumes)

        return self.__volumes

class SMVEVolumeCalculator(VolumeCalculatorBase):

    """
    Method to calculate the volume introduced by Srolovitz, Maeda, 
    Vitek and Egami.
    """

    def __init__(self, volume_setting):
        VolumeCalculatorBase.__init__(self, volume_setting)

    def prepare(self):
        self.__well2s = numpy.array(list(self.gen_wells()))**2

    def gen_wells(self):
        """Generate the well from radial distribution functions."""

        # get parameter from setting
        traj_parser = self._setting.traj_parser
        rmax     = self._setting.smve_rmax
        dr       = self._setting.smve_dr
        interval = self._setting.smve_interval
        incr     = self._setting.smve_increment

        # calculate radial distribution functions
        from calrdf import average_rdf
        rdfs = average_rdf(traj_parser, rmax=rmax, dr=dr, interval=interval,
                average=True, per_area=True) 

        # search 1st wells
        from search_well import gen_searched_wells
        for well_info in gen_searched_wells(rdfs, dr, increment=incr):
            tmp1, tmp2 ,well, tmp4 = well_info
            yield well

    def get_volume(self, crd):
        radii, volumes = self.get_volume_fort(crd)

        # for iatm_1, (radius, volume) in enumerate(zip(radii, volumes)):
        #     logger.info(iatm_1+1, radius, volume)

        return volumes

    def get_volume_py(self, crd):
        """Calculate and get the volume by python code."""
        radii = numpy.array( list(self.gen_radius(crd)) )
        volumes = 4.0*math.pi*radii**3 / 3.0
        return radii, volumes

    def gen_radius(self, crd):
        """Calculate a radius and volume on each atoms by wells."""
        # wells2 is powered by its self(wells).

        for iatm_1 in range(self._setting.natom):
            well2 = self.__well2s[iatm_1]
            num = 0.0 # numerator
            den = 0.0 # denominator

            for jatm_1 in range(self._setting.natom.natom):
                if iatm_1 == jatm_1: continue
                r_ij = crd[iatm_1] - crd[jatm_1]
                l_ij2 = numpy.dot(r_ij, r_ij)

                if l_ij2 > well2: continue

                l_ij = math.sqrt(l_ij2)

                num += 1.0/l_ij
                den += 1.0/l_ij2

            yield 0.5 * num / den

    def get_volume_fort(self, crd):
        """Calculate and get the volume by fortran code."""
        from lib_calvolume import calvolume
        radii, volumes = calvolume(crd, self.__well2s)
        return radii, volumes

import lib_voronoi as lib_voro
import lib_sink
class VoronoiVolumeCalculatorBase(VolumeCalculatorBase):
    
    """
    This class is the base class to use o calculate the volume obtained
    from voronoi polyhedra.
    Please beware because this method is time-consuming routine.
    """

    def __init__(self, vsetting):
        VolumeCalculatorBase.__init__(self, vsetting)

    @abstractmethod
    def preprocess_crd(self, crd):
        """Preprocess the coordinate by any way."""
        return

    @abstractmethod
    def get_enable_atoms(self, crd):
        """Get the boolean array that atoms are enable for voronoi method."""
        return

    def cal_volume(self, crd):
        """Calculate and get the volume by fortran code."""
        self.__radii, self.__volumes = self.cal_voronoi(crd)
        # radii, volumes = self.cal_voronoi_with_debug(crd)
        return self.__volumes

    def init_voronoi(self, crd):
        """Initialize voronoi calculation for one snapshop."""

        # preprocess
        new_crd = self.preprocess_crd(crd)

        # get enable atoms for current coordinate
        self.__is_enable_atoms = self.get_enable_atoms(new_crd)

        # get cutoff
        cutoff = self._setting.voronoi_cutoff

        # initialize the voronoi_one module
        lib_voro.voronoi_one.initialize(self.__is_enable_atoms, cutoff, 30.0)

        return new_crd, self.__is_enable_atoms

    def get_gvolume(self, crd=[]):
        nohyd = self._setting.voronoi_no_hydrogen
        if self._setting.group_method == 'united' and nohyd:
            gvolumes = []

            # for itar_1, iatm in enumerate(self._setting.target_atoms):
            #     if not self.__is_enable_atoms[iatm-1]: continue

            #     gvolumes.append( self.__volumes[itar_1] )

            for gname, iatoms in self._setting.gname_iatoms_pairs:
                gvol = None
                for iatm in iatoms:
                    if self.__is_enable_atoms[iatm-1]:
                        itar = self.get_itar_by_iatm(iatm)
                        gvol = self.__volumes[itar-1]
                        break

                if gvol is None:
                    msg = ('At gname = {} volume cannot be defined. '
                           'Please set the difinition of target_atoms or group '
                           'to the appropriate value.')
                    raise VolumeNotDefinedError( msg.format(gname) )
                else:
                    gvolumes.append( gvol )

            # write volumes
            self.write_group_volume(gvolumes)

            return gvolumes

        else:
            return VolumeCalculatorBase.get_gvolume(self, crd)
    
    def cal_voronoi(self, crd):
        """Calculate and get the volume by fortran code."""
        # print(1)
        try:
            # print(2)
            crd, enable_atoms = self.init_voronoi(crd)
            # print(3)
        except ValueError:
            # print(4)
            msg = ("When a solvated system is handled, "
                   "the keyword voronoi_solvation should be 'none'.")
            # print(5, msg)
            raise WurpException(msg)
            # print(6)
        # print(7)

        radii, volumes = lib_voro.cal_voronoi_all(crd,
                self._setting.target_atoms, enable_atoms)
        return radii, volumes

    def cal_voronoi_with_debug(self, crd):

        crd, enable_atoms = self.init_voronoi(crd)

        # For the atomic calculation, the case of no hydrogen gives
        #the default value of the hydrogen volume.
        hyd_volume = 8.0

        # give the coordinate to the voronoi_one module
        lib_voro.voronoi_one.crd = crd

        volumes   = []
        nnabs     = []
        nabs_list = []

        ntar = len(self._setting.target_atoms)

        for iatm in self._setting.target_atoms:

            if self.__is_enable_atoms[iatm-1]:
                volume = lib_voro.cal_voronoi_one(iatm)
                self.write_informations(lib_voro.voronoi_one)

                nnabs.append(lib_voro.voronoi_one.nnab)
                nabs_list.append(lib_voro.voronoi_one.nab_list)

            else:
                volume = hyd_volume
                nnabs.append(None)
                nabs_list.append(None)

            volumes.append( volume )

        self.write_summary(nnabs, nabs_list)

        radii = lib_voro.cal_radii_sphere(numpy.array(volumes))
        return radii, volumes

    def write_informations(self, mod):
        """Write out the information of one atom 
        obtained by Voronoi analysis."""

        # write out results
        logger.debug(' NUMBER OF NEIGHBOURS', mod.nface)
        logger.debug(' NEIGHBOUR LIST')

        logger.debug(' {:5s}   {:5s}'.format('ATOM', 'FACE'))
        logger.debug(' {:5s}   {:5s}   {:>36s}   {:>12s}'.format(
                'INDEX', 'EDGES', 'RELATIVE POSITION', 'DISTANCE'))

        can_fmt = " {:5s}   {:5s}   {:12.5f}{:12.5f}{:12.5f}   {:12.5f}"
        for ican_1 in range(mod.ncan):
            if mod.edges[ican_1] == 0: continue

            logger.debug(can_fmt.format(
                mod.can_to_iatm[ican_1], mod.edges[ican_1],
                mod.can_crd[ican_1, 0], mod.can_crd[ican_1, 1],
                mod.can_crd[ican_1, 2], sqrt(mod.can_len2[ican_1])
            ))

        logger.debug(' NUMBER OF EDGES ', mod.nedge)
        logger.debug(' NUMBER OF VERTICS ', mod.nver)
        logger.debug(' VERTEX LIST ')
        logger.debug(' {:>18s}   {:>36s}'.
                format('INDICES', 'RELATIVE POSITION'))

        for iver_1 in range(mod.nver):

            ican = mod.ver_to_3cans[iver_1, 0]
            jcan = mod.ver_to_3cans[iver_1, 1]
            kcan = mod.ver_to_3cans[iver_1, 2]

            # logger.debug(' {:6d}{:6d}{:6d}   {:12.5f}{12.5f}{12.5f})'.format(
            #     mod.can_to_iatm[ican-1], mod.can_to_iatm[jcan-1],
            #     mod.can_to_iatm[kcan-1], mod.ver_crd[iver_1, 0],
            #     mod.ver_crd[iver_1, 1],  mod.ver_crd[iver_1, 2]
            # ))
            logger.debug(' {:6}{:6}{:6}   {:12.5f}{:12.5f}{:12.5f}'.format(
                mod.can_to_iatm[ican-1], mod.can_to_iatm[jcan-1],
                mod.can_to_iatm[kcan-1], mod.ver_crd[iver_1, 0],
                mod.ver_crd[iver_1, 1],  mod.ver_crd[iver_1, 2]
            ))

    def write_summary(self, nnabs, nabs_list):
        """Write out the summary of Voronoi calculation."""

        # ** summary **
        logger.debug(' FINAL SUMMARY')
        logger.debug(' {:5s}   {:5s}   {}'.format(
            'INDEX', 'NABS', '... NEIGHBOURS INDICES' ))

        ncoord = 0
        for itar_1, (iatm, nnab, nab_list) in enumerate(
                zip(self._setting.target_atoms, nnabs, nabs_list)):

            if nnab is None:
                logger.debug( ' {:5} is not calculated.'.format(iatm) )
                continue

            ncoord = ncoord + nnab

            logger.debug( ' {:5}   {:5}   {}'.
                    format( iatm, nnab, nab_list[:nnab]))

            # ** check that if jatm is a neighbour of iatm **
            # ** then iatm is also a neighbour of jatm     **

            # for inab_1 in zip(self.__target_atoms, ange(nnabs[iatm-1]):

            # for jatm, nnab2 in zip(self.__target_atoms, ange(nnabs[iatm-1]):

            for jatm in nab_list[:nnab]:
                jtar = self.get_itar_by_iatm(jatm)
                if jtar == 0: continue
                found = False

                nab_j_list = nabs_list[jtar-1]
                nnab_j     = nnabs[jtar-1]

                for katm in nab_j_list[:nnab_j]:
                    # exit if there is iatm in the jatm-list at least.
                    if found: break

                    found = iatm == katm

                else:
                    logger.debug(' {:5} is not a neighbour of {:5}'.format(
                        iatm, jatm) )

            # for jatm, nnab2 in zip(self.__target_atoms, ange(nnabs[iatm-1]):
            #     jatm = nab_list[inab_1, iatm_1]
            #     found = False

            #     for jnab_1 in range(nnab[jatm-1]):
            #         # exit if there is iatm in the jatm-list at least.
            #         if found: break

            #         found = iatm == nab_list[jnab_1, jatm-1]

            #     else:
            #         logger.debug(' {:5d} is not a neighbour of {:5d}'.format(
            #             iatm, jatm) )


        coord = float(ncoord/float(self._setting.natom))
        logger.debug(' average coordination number = {:10.5f}'.format(coord))

class VoronoiVolumeCalculator(VoronoiVolumeCalculatorBase):
    
    """
    This class will be used if voronoi_solvation is 'none'.
    """

    def __init__(self, vsetting):
        self.__is_enable_atoms = None
        VoronoiVolumeCalculatorBase.__init__(self, vsetting)

    def prepare(self):
        self.__is_enable_atoms = self.get_enable_atoms()

    def preprocess_crd(self, crd):
        return crd

    def get_enable_atoms(self, crd=[]):
        if self.__is_enable_atoms is None:
            if self._setting.voronoi_no_hydrogen:
                is_enable_atoms = numpy.array(
                        [ elem!='H' for elem in self._setting.elems ] )

            else:
                is_enable_atoms = numpy.ones(
                        [self._setting.natom], numpy.bool )

            self.__is_enable_atoms = is_enable_atoms

        return self.__is_enable_atoms

class VoronoiVolumeCalculatorWithSolvation(VoronoiVolumeCalculatorBase):
    
    """
    This class will be used if voronoi_solvation is not 'none'.
    """

    def __init__(self, vsetting):
        self.__water_crd = None
        self.__is_system_atoms = None
        self.__is_enable_system_atoms = None
        self.__itraj = 0
        self.__count = 0
        self.__do_write_crd = False
        self.__fn_base = None
        self.__fn_ext = None
        VoronoiVolumeCalculatorBase.__init__(self, vsetting)

    def prepare(self):

        # parameter
        nohyd  = self._setting.voronoi_no_hydrogen
        stype  = self._setting.voronoi_solvation

        # load the water molucules data from pdb file
        mod_dir = os.path.dirname(__file__)
        type_to_fns = {
                'RANDOM20' : os.path.join(mod_dir, './random20.pdb.gz'),
                'NPT20'    : os.path.join(mod_dir, './npt20.pdb.gz'   ),
                'NPT25'    : os.path.join(mod_dir, './npt25.pdb.gz'   ),
                'NPT30'    : os.path.join(mod_dir, './npt30.pdb.gz'   ),
        }
        self.__water_crd = self.parse_water_pdb( type_to_fns[stype] )

        # make is_enable_system_atoms
        if nohyd:
            self.__is_enable_system_atoms = numpy.array(
                    [ elem!='H' for elem in self._setting.elems ] )

        else:
            self.__is_enable_system_atoms = numpy.ones(
                    [self._setting.natom], numpy.bool)

        # output solvation coordinate
        filename = self._setting.voronoi_output_solvation_file
        if filename != '':
            self.__do_write_crd = True
            self.__fn_base, self.__fn_ext = os.path.splitext(filename)

    def preprocess_crd(self, crd):
        new_crd = self.solvate_crd(crd)
        if self.__do_write_crd:
            self.write_crd(new_crd)
        return new_crd

    def get_enable_atoms(self, crd):
        if self._setting.voronoi_no_hydrogen:
            is_enable_atoms = numpy.ones( [len(crd)], numpy.bool)
            natom = len(self.__is_enable_system_atoms)
            is_enable_atoms[:natom] = self.__is_enable_system_atoms

        else:
            is_enable_atoms = numpy.ones( [len(crd)], numpy.bool)

        return is_enable_atoms

    def parse_water_pdb(self, filename):
        """Parse the water solvent pdb and get the coordinates and
        elements from it."""

        atoms = list(PDB.System(filename))

        water_crd   = []
        if self._setting.voronoi_no_hydrogen:
            for atom in atoms:
                if atom.element == 'H': continue
                water_crd.append( atom.pos )

        else:
            for atom in atoms:
                water_crd.append( atom.pos )

        return numpy.array( water_crd )

    def solvate_crd(self, crd):
        """Return the coordinate that sunk the system into watar molecules."""
        probe = self._setting.voronoi_probe_length
        if self._setting.voronoi_no_hydrogen:
            solv_crd, natom_solv = lib_sink.sink_nohyd(
                    crd, self.__water_crd, probe )
        else:
            solv_crd, natom_solv = lib_sink.sink( crd, self.__water_crd, probe )
        return solv_crd[:natom_solv]

    def write_crd(self, crd):

        line_template = (
                'ATOM  {id:>5} {name:<4} {rname:>3} {cname:>1}{rid:>4}'
            '    {x:8.3f}{y:8.3f}{z:8.3f}{occ:6.2f}{bfact:6.2f}'
            '{other:>4}       {elem:<2} '
        )

        names = self._setting.names
        resnames = self._setting.resnames
        rids = self._setting.rids
        elems = self._setting.elems

        occ   = 1.0
        bfact = 0.0
        cname = ''
        other = ''

        pdblines = []

        # for system
        for iatm_1, (name, rname, rid, pos, elem) in enumerate(
                        zip(names, resnames, rids, crd, elems) ):

            pdbline = line_template.format(
                id=iatm_1+1, name=name, rname=rname, rid=rid, cname=cname,
                x=pos[0], y=pos[1], z=pos[2], occ=occ, bfact=bfact,
                other=other, elem=elem )

            pdblines.append( pdbline )

        pdblines.append( 'TER')
        # for solvent
        iatm_end = iatm_1+1
        rid_end = rid

        if self._setting.voronoi_no_hydrogen:
            natom_sol = len(crd) - len(names) 
            nres_sol = natom_sol

            for index in range(nres_sol):
                iatm = iatm_end + index + 1
                rid  = rid_end + index + 1

                pos = crd[iatm-1]
                pdbline = line_template.format(
                    id=iatm, name='O', rname='WAT', rid=rid, cname=cname,
                    x=pos[0], y=pos[1], z=pos[2], occ=occ, bfact=bfact,
                    other=other, elem='O' )
                pdblines.append( pdbline )
                pdblines.append( 'TER')

        else:
            natom_sol = len(crd) - len(names) 
            nres_sol = natom_sol/3

            for index in range(nres_sol):
                iatm = iatm_end + 3*index + 1
                rid  = rid_end + index + 1

                pos = crd[iatm-1]
                pdbline = line_template.format(
                    id=iatm, name='O', rname='WAT', rid=rid, cname=cname,
                    x=pos[0], y=pos[1], z=pos[2], occ=occ, bfact=bfact,
                    other=other, elem='O' )
                pdblines.append( pdbline )

                pos = crd[iatm-1+1]
                pdbline = line_template.format(
                    id=iatm+1, name='H1', rname='WAT', rid=rid, cname=cname,
                    x=pos[0], y=pos[1], z=pos[2], occ=occ, bfact=bfact,
                    other=other, elem='H' )
                pdblines.append( pdbline )

                pos = crd[iatm-1+2]
                pdbline = line_template.format(
                    id=iatm+2, name='H2', rname='WAT', rid=rid, cname=cname,
                    x=pos[0], y=pos[1], z=pos[2], occ=occ, bfact=bfact,
                    other=other, elem='H' )
                pdblines.append( pdbline )

                pdblines.append( 'TER')
    
        self.__count += 1
        filename = '{}{:05}{}'.format(
                self.__fn_base, self.__count, self.__fn_ext )

        file = open(filename, 'w')
        for line in pdblines:
            file.write(line + '\n')
        file.close()


class Volume1(VolumeCalculatorBase):
    
    """
    Method to use 1 as volume values for the group calculations.
    """

    def __init__(self, volume_setting):
        VolumeCalculatorBase.__init__(self, volume_setting)

    def prepare(self):
        self.__volumes  = numpy.ones( [len(self._setting.target_atoms)] )
        self.__gvolumes = numpy.ones( [len(self._setting.gname_iatoms_pairs)] )

    def cal_volume(self, crd=[]):
        """Return 1.0."""
        return self.__volumes

    def get_gvolume(self, crd=[]):
        """Return 1.0."""
        self.write_group_volume(self.__gvolumes)
        return self.__gvolumes

class OuterVolumeFetcher(VolumeCalculatorBase):
    
    """
    A class that performed the method to get outer volume trajectory file
    already calculeted by any methods.
    """

    end_flag = '#END'
    comment  = '#'

    def __init__(self, volume_setting):
        traj_fn = volume_setting.atomic_trajectory_file
        self.__file = open(traj_fn, 'r')

        grp_traj_fn = volume_setting.group_trajectory_file
        if grp_traj_fn == '':
            self.__gfile = None
        else:
            self.__gfile = open(grp_traj_fn, 'r')

        VolumeCalculatorBase.__init__(self, volume_setting)

    def prepare(self):
        self.__ngrp = len(self._setting.gname_iatoms_pairs)
        self.__ntar = len(self._setting.target_atoms)

    def cal_volume(self, crd):
        try:
            ids, names, vols = self.parse_atomic().next()

            if len(vols) != self.__ntar:
                msg = 'The number of target atoms: {}, but in volume trajectory: {}'
                raise NumberOfTargetError(
                        msg.format(self.__ntar, len(vols)) )

            return vols
        except StopIteration:
            return None

    def get_gvolume(self, crd=[]):
        if self.__gfile is None:
            return VolumeCalculatorBase.get_gvolume(self, crd)
        else:
            try:
                ids, names, vols = self.parse_group().next()

                if len(vols) != self.__ngrp:
                    msg = 'The number of groups: {}, but in volume trajectory: {}'
                    raise NumberOfTargetError(
                            msg.format(self.__ngrp, len(vols)) )

                self.write_group_volume(vols)

                return vols
            except StopIteration:
                return None

    def parse_atomic(self):
        return self.parse_trajectory(self.__file)

    def parse_group(self):
        return self.parse_trajectory(self.__gfile)

    def parse_trajectory(self, file):

        lines = []
        for line in file:
            if line.startswith(self.end_flag):
                yield self.parse_snapshop(lines)
                lines = []
            elif line.startswith('#'):
                pass
            else:
                lines.append(line)

        self.__file.close()

    def parse_snapshop(self, lines, other=False):
        natom = len(lines)
        ids = numpy.zeros((natom),dtype=numpy.int)
        names = numpy.zeros((natom), dtype=numpy.str)
        vols = numpy.zeros((natom))

        for iatm, line in enumerate(lines):
            id, name, vol = line.split()
            ids[iatm] = id
            names[iatm] = name
            vols[iatm] = vol

        return ids, names, vols

        for i, atom in enumerate(self.__system.atoms):

            cname = atom.chain.name if atom.chain else ' '
            if atom.chain:
                if not first:
                    if cname != prev_cname:
                        pdblines.append('TER')
                        prev_cname = cname
                else:
                    first = False
            # else:
            #     if rid != prev_rid:
            #         pdblines.append('TER')
            #         prev_rid = rid

            # id = atom.id
            id = i+1
            rid   = atom.residue.id
            rname = atom.residue.name
            x, y, z = atom.pos
            prev_cname = cname
            # prev_rid = rid

            # format the atom name
            aname = atom.name if atom.name[0].isdigit() else ' ' + atom.name

            line_template = (
                'ATOM  {id:>5} {name:<4} {rname:>3} {cname:>1}{rid:>4}'
                '    {x:8.3f}{y:8.3f}{z:8.3f}{occ:6.2f}{bfact:6.2f}'
                '{other:>4}       {elem:<2} '
            )
            pdbline = line_template.format(
                id=id, name=aname, rname=rname, rid=rid, cname=cname,
                x=x, y=y, z=z, occ=atom.occ, bfact=atom.bfact,
                other=atom.other, elem=atom.elem )

            if term_resname:
                if term_resname != rname:
                    pdblines.append('TER')
                    term_resname = ""

            pdblines.append(pdbline)

            if atom.name == 'OXT':
                term_resname = rname
                # pdblines.append('TER')

        else:
            if pdblines != []:
                pdblines.append('TER')
            pass

        return '\n'.join(pdblines)


def pdb_test():

    def save_merged_pdb(pdb_fn, bfactors):

        atoms = list(PDB.System(pdb_fn))
        for atom, bfact in zip(atoms, bfactors):
            atom.bfactor = bfact

        return atoms

    def save_merged_pdb_three(pdb_fn, bfactors):

        atoms = System(PDB.System(pdb_fn))
        for atom, bfact in zip(atoms, bfactors):
            if bfact < 1.0:
                bfact = 0.0
            elif 1.0 <= bfact <= 10.0:
                bfact = 10.0
            elif 10.0 < bfact:
                bfact = 50.0
            else:
                pass

            atom.bfactor = bfact

        return atoms

    def save_pdb(atoms, pdb_fn):

        with  open(pdb_fn, 'w') as pdb_file:
            for line in PDB.gen_pdbline(atoms):
                pdb_file.write(line + "\n")



"""clog - Defines functions of what to print in the logs.

If log_level is set to 1 (WARN):
  only warning messages will be printed.
If log level is set to 2 (default) (INFO):
  warning and info messages will be printed.
If log level is set to 4 (DEBUG):
  warning, info and debug messages will be printed.
"""

from __future__ import print_function


WARN = 1
INFO = 2
DEBUG = 4 

print_function = print
log_level = INFO

title_fmt = '\n' + 80*'-' + '\n' + 8*' '+ '{}' '\n' + 80*'-' + '\n'
title_fmt_info = '\n' + 80*'-' + '\n' + '{:>6}. {}' '\n' + 80*'-' + '\n'
count = 0
log_frequency = 1
cur_istep = 0

def set_log_frequency(frequency):
    global log_frequency
    log_frequency = frequency

def set_curstep(istp):
    global cur_istep 
    cur_istep = istp

def print(*args, **kwds):
    print_function(*args, **kwds)

#Debug functions

def debug(*args, **kwds):
    if log_level >= DEBUG: # log_level = 4
        print_function(*args, **kwds)

def debug_cycle(*args, **kwds):
    """Log informations depending on log_frequency."""
    if cur_istep in [0,1] or cur_istep%log_frequency==0:
        debug(*args, **kwds)

def debug_title(title):
    debug(title_fmt.format(title))

def is_debug():
    return log_level == DEBUG

#Info functions

def info(*args, **kwds):
    if log_level >= INFO:
        print_function(*args, **kwds)

def info_cycle(*args, **kwds):
    """Log informations depending on log_frequency."""
    if cur_istep in [0,1] or cur_istep%log_frequency==0:
        info(*args, **kwds)

def info_title(title):
    global count
    count += 1
    info(title_fmt_info.format(count, title))

def is_info():
    return log_level == INFO

#Warn functions

def warn(*args, **kwds):
    if log_level >= WARN:
        print_function(*args, **kwds)

def warn_title(title):
    warn(title_fmt.format(title))
def is_warn():
    return log_level == WARN



from __future__ import print_function

import os, sys
import time
topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)
import utility
import numpy

# wurp modules
import clog as logger

import base

################################################################################
class StressCurrentCalculator(base.CurrentCalculator):

    """
    A class to caluculate the stress current.
    """

    def __init__(self):
        base.CurrentCalculator.__init__(self)

    def prepare(self, *args, **kwds):
        """Prepare the calculations of all snapshots."""
        base.CurrentCalculator.prepare(self, *args, **kwds)
        
        self.scal = StressTensor( self.get_target_atoms(),
                self.get_iatm_to_igrp(), self.get_bonded_pairs())

    def cal_kinetic(self, vel):

        masses   = self.get_topology().get_atom_info()['masses']
        volumes  = self.get_volume()
        gvolumes = self.get_gvolume()

        t0 = time.time()

        current, current_grp, current_grp_inner = self.scal.cal_kinetic(
                vel, masses, volumes, gvolumes)

        t1 = time.time()
        self.store_time('kinetic current', t1-t0)

        return current, current_grp, current_grp_inner

    def cal_bonded(self, crd, bond_type):
        """Calculate the current for one of bonded types."""
        tbcal = self.get_tbforce()

        # calculate two-body force of each bonded interactions
        t0 = time.time()
        tbfs_info = tbcal.cal_bonded(bond_type)
        tbfs      = tbfs_info['tbforces']

        # get valume
        volumes   = self.get_volume()
        gvolumes  = self.get_gvolume()

        # calculate current
        t1 = time.time()
        current, current_grp, current_grp_inner = self.scal.cal_bonded(
                crd, tbfs, volumes, gvolumes)

        # log the elasped times
        t2 = time.time()
        self.store_time(bond_type+' pairwise', t1-t0)
        self.store_time(bond_type+' current' , t2-t1)

        return current, current_grp, current_grp_inner

    def cal_coulomb(self, crd):
        """Calculate the current for coulomb components."""

        table     = self.get_interact_table()
        type_func = self.get_tbforce().cal_coulomb
        cutoff    = self.get_setting().wurp.coulomb_cutoff_length
        volumes   = self.get_volume()
        gvolumes  = self.get_gvolume()

        t0 = time.time()
        gen_tbfs = ( type_func(t)['tbforces'] for t in table )
        current, current_grp, current_grp_inner  = self.scal.cal_nonbonded(
                crd, gen_tbfs, volumes, table, gvolumes)

        t1 = time.time()
        dt_current = self.scal.dt

        self.store_time('coulomb pairwise' , t1 - t0 - dt_current)
        self.store_time('coulomb current' , dt_current)

        return current, current_grp, current_grp_inner

    def cal_vdw(self, crd):
        """Calculate the current for vdW components."""

        table     = self.get_interact_table()
        type_func = self.get_tbforce().cal_vdw
        cutoff    = self.get_setting().wurp.vdw_cutoff_length
        volumes   = self.get_volume()
        gvolumes  = self.get_gvolume()

        t0 = time.time()

        gen_tbfs = ( type_func(t)['tbforces'] for t in table )
        current, current_grp, current_grp_inner  = self.scal.cal_nonbonded(
                crd, gen_tbfs, volumes, table, gvolumes)

        t1 = time.time()
        dt_current = self.scal.dt

        self.store_time('vdw pairwise' , t1 - t0 - dt_current)
        self.store_time('vdw current' , dt_current)

        return current, current_grp, current_grp_inner


import lib_current
class StressTensor:

    def __init__(self, target_atoms, iatm_to_igrp, bonded_pairs):
        lib_current.bonded.initialize(target_atoms, iatm_to_igrp, bonded_pairs)
        lib_current.nonbonded.initialize(target_atoms, iatm_to_igrp)
        lib_current.kinetic.initialize(target_atoms, iatm_to_igrp)

    def cal_bonded(self, crd, tbfs, volumes, gvolumes=[]):
        """Calculate the current due to bonded potentials."""
        m_bond = lib_current.bonded

        # calculate
        m_bond.cal_bonded(crd, tbfs, volumes, gvolumes)

        # get current by copy
        cur_atm = m_bond.current.copy()
        cur_inn = m_bond.current_inn.copy()
        cur_out = m_bond.current_out.copy()
        return cur_atm, cur_inn, cur_out

    def cal_nonbonded(self, crd, gen_tbfs, volumes, table, gvolumes=[]):
        """Calculate the current due to nonbonded potentials."""
        t0 = time.time()

        m_non = lib_current.nonbonded

        # initialize
        m_non.init_cal(crd, volumes, gvolumes)

        # calculate
        t_total = time.time() - t0
        for t, tbfs in zip(table, gen_tbfs):
            t1 = time.time()
            m_non.cal_nonbonded(tbfs, t)
            t_total += time.time() - t1

        t2 = time.time()

        # get current by copy
        cur_atm = m_non.current.copy()
        cur_inn = m_non.current_inn.copy()
        cur_out = m_non.current_out.copy()

        t_total += time.time() - t2
        self.dt = t_total

        return cur_atm, cur_inn, cur_out

    def cal_kinetic(self, vel, masses, volumes, gvolumes=[]):
        """Calculate the current due to kinetic terms."""
        m_kin = lib_current.kinetic

        # convert the unit from g*[A/fs]^2 to [kcal/mol]
        s = 10.0**4 / 4.184

        # calculate
        m_kin.cal_kinetic(vel, masses, volumes, gvolumes)

        # get current
        return m_kin.current*s, m_kin.current_inn*s, m_kin.current_out*s

    def cal_bonded_old(self, crd, tbfs, volumes):
        """Calculate the current due to bonded potentials."""
        return lib_current.cal_bonded(crd, tbfs, volumes)

    def cal_nonbonded_old(self, crd, gen_tbfs, volumes, table):
        """Calculate the current due to nonbonded potentials."""

        # initialize
        lib_current.nonbond.initialize(crd, volumes)

        for t, tbfs in zip(table, gen_tbfs):
            lib_current.nonbond.cal_nonbonded(tbfs, t)

        return lib_current.nonbond.current.copy()

    def cal_kinetic_old(self, vel, volumes, masses):
        ps = [m*v/vol for v, vol, m in zip(vel, volumes, masses)]
        return utility.tensor(numpy.array(ps), vel)



################################################################################

# TODO: Theory is yet not created ...
class EnergyCurrentCalculator(base.CurrentCalculator):

    def cal_bonded(self):
        """Calculate the energy current for the bonded potential terms."""
        
        # calculate two-body force of bonded interactions
        self.__tbf.cal_bonded()

        # get two-body forces
        bonded_tbfs = self.__tbf.get_bonded_tb_forces()
        bonded14_tbfs = self.__tbf.get_bonded14_tb_forces()

        # initialize
        current_pot = numpy.zeros([self.__natom+1, 3])

        # bonded two-body force
        for iatm, jatm, f_ij in bonded_tbfs.items():
            v_ij = 0.5 * (vel[iatm] + vel[jatm])
            f_ij_v_ij = f_ij * v_ij
            current_pot[iatm] += f_ij_v_ij / volume_ij
            current_pot[jatm] += f_ij_v_ij / volume_ji

        # bonded14 two-body force
        for iatm, jatm, f_ij in bonded14_tbfs.items():
            v_ij = 0.5 * (vel[iatm] + vel[jatm])
            f_ij_v_ij = f_ij * v_ij
            current_pot[iatm] += f_ij_v_ij / volume_ij
            current_pot[jatm] += f_ij_v_ij / volume_ji

        return current_pot

    #TODO: volume calculation
    def _cal_nonbond(self):
        current_pot = numpy.zeros([self.__natom+1, 3, 3])

        for iatm, jatm_ptr, f_ijs in gen_tbforces_each_atom:

            for j, f_ij in enumerate(f_ijs):
                jatm = jatm_ptr + j

                v_ij = 0.5 * (vel[iatm] + vel[jatm])
                f_ij_v_ij = f_ij * v_ij
                current_pot[iatm] += f_ij_v_ij / volume_ij
                current_pot[jatm] += f_ij_v_ij / volume_ji

        return current_pot
                

if __name__ == '__main__':
    cal = StressCurrentCalculator()
    # table = cal.make_interaction_table(natom=5)



#! /usr/bin/env python2

import sys, os

def parse_line(line):
    cols = line.split()
    don_line, acc_line, rest = cols[0], cols[1], cols[2:]

    rid_don, rname_don = don_line.split('_')
    rid_don = int(rid_don)
    rid_acc, rname_acc = acc_line.split('_')
    rid_acc = int(rid_acc)

    return rid_don, rid_acc, rname_don, rname_acc, rest

if __name__ == '__main__':

    # arguments
    if len(sys.argv) == 2:
        sort_method = sys.argv[1]
    else:
        print('{} < <ec_file> <sort method>'.format(sys.argv[0]))
        exit()

    ec_file = sys.stdin

    # remove the only space line and the comment line
    line_iter = ( line.strip() for line in ec_file
            if not line.isspace() if not line.startswith('#') )

    # store
    rkey_to_count = {}
    for line in line_iter:
        rid1, rid2, rname1, rname2, rest = parse_line(line)

        ec = float(rest[0])

        count1 = rkey_to_count.get((rid1, rname1))
        rkey_to_count[rid1, rname1] = count1+1 if count1 else 1

        count2 = rkey_to_count.get((rid2, rname2))
        rkey_to_count[rid2, rname2] = count2+1 if count2 else 1

    # sort
    # seq, count, rev_count
    if sort_method == 'seq':
        sorted_items = sorted( rkey_to_count.items(),
            key=lambda rkey: rkey[0][0], reverse=False)
    elif sort_method == 'count':
        sorted_items = sorted( rkey_to_count.items(),
            key=lambda rkey: rkey[1], reverse=True)
    elif sort_method == 'rev_count':
        sorted_items = sorted( rkey_to_count.items(),
            key=lambda rkey: rkey[1], reverse=False)
    else:
        sorted_items = rkey_to_count.items()

    # print
    print('#{:>8}{:>5}   {:>2}'.format('resname', 'rid', 'count'))
    for (rid, rname), count in sorted_items:
        print(' {:>8}{:>5}   {:>2}'.format(rname, rid, count))




from __future__ import print_function

import os, sys
import time
import numpy

# wurp modules
topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)
import utility
import clog as logger

import base
################################################################################
class EnergyFluxCalculator(base.FluxCalculator):

    """
    This class calculates the energy flux.
    """

    def __init__(self):
        base.FluxCalculator.__init__(self)

    def prepare(self, *args, **kwds):
        """Prepare the calculations of all snapshots."""
        base.FluxCalculator.prepare(self, *args, **kwds)

        ftype = self.get_setting().wurp.flux_grain
        if ftype == 'atom':
            flag_atom  = True
            flag_group = False
        elif ftype == 'group':
            flag_atom  = False
            flag_group = True
        elif ftype == 'both':
            flag_atom  = True
            flag_group = True
        else:
            pass

        self.fcal = EnergyFlux( self.get_target_atoms(),
                self.get_iatm_to_igrp(), self.get_bonded_pairs(),
                flag_atom, flag_group)

    def cal_bonded(self, crd, vel, bond_type):
        """Calculate the energy flux for the bonded term."""

        tbcal = self.get_tbforce()

        # calculate two-body force of bonded interactions
        t0 = time.time()
        tbfs_info = tbcal.cal_bonded(bond_type)
        tbfs      = tbfs_info['tbforces']


        # calculate current
        t1 = time.time()
        flux_atm, flux_grp = self.fcal.cal_bonded(vel, tbfs)

        # log the elasped times
        t2 = time.time()
        self.store_time(bond_type+' pairwise', t1-t0)
        self.store_time(bond_type+' flux' , t2-t1)

        return flux_atm, flux_grp

    def cal_coulomb(self, crd, vel):
        """Calculate the energy flux for the coulomb term."""

        table     = self.get_interact_table()
        type_func = self.get_tbforce().cal_coulomb
        cutoff    = self.get_setting().wurp.coulomb_cutoff_length

        t0 = time.time()
        gen_tbfs = ( type_func(t)['tbforces'] for t in table )
        flux_atm, flux_grp = self.fcal.cal_nonbonded( vel, gen_tbfs, table )

        t1 = time.time()
        dt_flux = self.fcal.dt

        self.store_time('coulomb pairwise' , t1 - t0 - dt_flux)
        self.store_time('coulomb flux' , dt_flux)

        return flux_atm, flux_grp

    def cal_vdw(self, crd, vel):
        """Calculate the energy flux for the vdW term."""

        table     = self.get_interact_table()
        type_func = self.get_tbforce().cal_vdw
        cutoff    = self.get_setting().wurp.vdw_cutoff_length

        t0 = time.time()
        gen_tbfs = ( type_func(t)['tbforces'] for t in table )
        flux_atm, flux_grp = self.fcal.cal_nonbonded( vel, gen_tbfs, table )

        t1 = time.time()
        dt_flux = self.fcal.dt

        self.store_time('vdw pairwise' , t1 - t0 - dt_flux)
        self.store_time('vdw flux' , dt_flux)

        return flux_atm, flux_grp


################################################################################
import lib_flux
class EnergyFlux:

    def __init__(self, target_atoms, iatm_to_igrp, bonded_pairs,
                       flag_atm=True, flag_grp=True):
        self.__flag_atm = flag_atm
        self.__flag_grp = flag_grp

        lib_flux.bonded.initialize( target_atoms, iatm_to_igrp,
                bonded_pairs, flag_atm, flag_grp)
        lib_flux.nonbonded.initialize( target_atoms, iatm_to_igrp,
                flag_atm, flag_grp)

    def cal_bonded(self, vel, tbfs):
        """Calculate the flux due to bonded potentials."""
        m_bond = lib_flux.bonded

        # calculate
        m_bond.cal_bonded(vel, tbfs)

        # get current by copy
        # flux_atm = m_bond.flux_atm if self.__flag_atm else None
        # flux_grp = m_bond.flux_grp if self.__flag_grp else None
        flux_atm = m_bond.flux_atm.copy() if self.__flag_atm else None
        flux_grp = m_bond.flux_grp.copy() if self.__flag_grp else None
        return flux_atm, flux_grp

    def cal_nonbonded(self, vel, gen_tbfs, table):
        """Calculate the flux due to nonbonded potentials."""
        t0 = time.time()
        m_non = lib_flux.nonbonded

        # initialize
        m_non.init_cal(vel)

        # calculate
        t_total = time.time() - t0
        tt = 0.0
        for t, tbfs in zip(table, gen_tbfs):
            t1 = time.time()
            m_non.cal_nonbonded(tbfs, t)
            t_total += time.time() - t1
            tt += time.time() - t1
            # print('flux body loop: ', time.time()-t1)

        # print('flux body:', tt)
        t2 = time.time()

        # get current by copy
        # flux_atm = m_non.flux_atm if self.__flag_atm else None
        # flux_grp = m_non.flux_grp if self.__flag_grp else None
        flux_atm = m_non.flux_atm.copy() if self.__flag_atm else None
        flux_grp = m_non.flux_grp.copy() if self.__flag_grp else None

        t_total += time.time() - t2
        self.dt = t_total

        return flux_atm, flux_grp

################################################################################
################################## HEAT FLUX ###################################

class HeatFluxCalculator(base.FluxCalculator):

    """
    This class calculates the heat flux.
    """

    def __init__(self):
        base.FluxCalculator.__init__(self)

    def prepare(self, *args, **kwds):
        """Prepare the calculations of all snapshots."""
        base.FluxCalculator.prepare(self, *args, **kwds)

        ftype = self.get_setting().wurp.flux_grain
        if ftype == 'atom':
            flag_atom  = True
            flag_group = False
        elif ftype == 'group':
            flag_atom  = False
            flag_group = True
        elif ftype == 'both':
            flag_atom  = True
            flag_group = True
        else:
            pass

        self.fcal = HeatFlux( self.get_target_atoms(),
                self.get_iatm_to_igrp(), self.get_bonded_pairs(),
                flag_atom, flag_group)

    def cal_bonded(self, crd, vel, bond_type):
        """Calculate the energy flux for the bonded term."""

        tbcal = self.get_tbforce()

        # calculate two-body force of bonded interactions
        t0 = time.time()
        tbfs_info = tbcal.cal_bonded(bond_type)
        tbfs      = tbfs_info['tbforces']
        displacement = tbfs_info['displacement']

        # calculate current
        t1 = time.time()
        flux_atm, flux_grp = self.fcal.cal_bonded(vel, tbfs, displacement)

        # log the elasped times
        t2 = time.time()
        self.store_time(bond_type+' pairwise', t1-t0)
        self.store_time(bond_type+' flux' , t2-t1)

        return flux_atm, flux_grp

    def cal_coulomb(self, crd, vel):
        """Calculate the energy flux for the coulomb term."""

        table     = self.get_interact_table()
        type_func = self.get_tbforce().cal_coulomb
        cutoff    = self.get_setting().wurp.coulomb_cutoff_length

        t0 = time.time()
        gen_tbfs = (type_func(t)['tbforces'] for t in table)
        gen_displacement = ( type_func(t)['displacement'] for t in table )

        flux_atm, flux_grp = self.fcal.cal_nonbonded( vel, gen_tbfs, table,
                                                      gen_displacement )

        t1 = time.time()
        dt_flux = self.fcal.dt

        self.store_time('coulomb pairwise' , t1 - t0 - dt_flux)
        self.store_time('coulomb flux' , dt_flux)

        return flux_atm, flux_grp

    def cal_vdw(self, crd, vel):
        """Calculate the energy flux for the vdW term."""

        table     = self.get_interact_table()
        type_func = self.get_tbforce().cal_vdw
        cutoff    = self.get_setting().wurp.vdw_cutoff_length

        t0 = time.time()
        gen_tbfs = ( type_func(t)['tbforces'] for t in table )
        gen_displacement = ( type_func(t)['displacement'] for t in table )

        flux_atm, flux_grp = self.fcal.cal_nonbonded( vel, gen_tbfs, table,
                                                      gen_displacement )

        t1 = time.time()
        dt_flux = self.fcal.dt

        self.store_time('vdw pairwise' , t1 - t0 - dt_flux)
        self.store_time('vdw flux' , dt_flux)

        return flux_atm, flux_grp


################################################################################
import lib_hflux
class HeatFlux:

    def __init__(self, target_atoms, iatm_to_igrp, bonded_pairs,
                       flag_atm=True, flag_grp=True):
        self.__flag_atm = flag_atm
        self.__flag_grp = flag_grp

        lib_hflux.bonded.initialize( target_atoms, iatm_to_igrp,
                bonded_pairs, flag_atm, flag_grp)
        lib_hflux.nonbonded.initialize( target_atoms, iatm_to_igrp,
                flag_atm, flag_grp)

    def cal_bonded(self, vel, tbfs, displ):
        """Calculate the flux due to bonded potentials."""
        m_bond = lib_hflux.bonded
        # calculate
        m_bond.cal_bonded(vel, tbfs, displ)

        # get current by copy
        # flux_atm = m_bond.flux_atm if self.__flag_atm else None
        # flux_grp = m_bond.flux_grp if self.__flag_grp else None
        hflux_atm = m_bond.hflux_atm.copy() if self.__flag_atm else None
        hflux_grp = m_bond.hflux_grp.copy() if self.__flag_grp else None
        return hflux_atm, hflux_grp

    def cal_nonbonded(self, vel, gen_tbfs, table, gen_displ):
        """Calculate the flux due to nonbonded potentials."""
        t0 = time.time()
        m_non = lib_hflux.nonbonded

        # initialize
        m_non.init_cal(vel)

        # calculate
        t_total = time.time() - t0
        tt = 0.0
        for t, tbfs, displ in zip(table, gen_tbfs, gen_displ):
            t1 = time.time()
            m_non.cal_nonbonded(tbfs, t, displ)
            t_total += time.time() - t1
            tt += time.time() - t1
            # print('flux body loop: ', time.time()-t1)

        # print('flux body:', tt)
        t2 = time.time()

        # get current by copy
        # flux_atm = m_non.flux_atm if self.__flag_atm else None
        # flux_grp = m_non.flux_grp if self.__flag_grp else None
        hflux_atm = m_non.hflux_atm.copy() if self.__flag_atm else None
        hflux_grp = m_non.hflux_grp.copy() if self.__flag_grp else None

        t_total += time.time() - t2
        self.dt = t_total

        return hflux_atm, hflux_grp




################################################################################
#TODO
class StressFluxCalculator(base.FluxCalculator):

    #TODO: volume calculation
    def cal_bonded_flux(self):
        """Calculate the stress flux for the bonded terms."""

        # calculate two-body force of bonded interactions
        self.__tbf.cal_bonded()

        # get two-body forces
        bonded_tbfs = self.__tbf.get_bonded_tb_forces()
        bonded14_tbfs = self.__tbf.get_bonded14_tb_forces()

        # initialize
        group_names = self.get_groupnames()
        flux_pot = {}
        for gname_i in group_names:
            for gname_j in group_names:
                ts = numpy.zeros([3,3])
                flux_pot[gname_i, gname_j] = ts

        # bonded two-body force
        for iatm, jatm, f_ij in bonded_tbfs.items():
            gname_i = self.__iatm_to_groups[iatm]
            gname_j = self.__iatm_to_groups[jatm]

            r_ij = crd[iatm] - crd[jatm]
            # f_ij_r_ij = utility.tensor1(f_ij, r_ij)
            flux_pot[gname_i, gname_j] += f_ij_r_ij / volume_ij
            flux_pot[gname_j, gname_i] += f_ij_r_ij / volume_ji

        # bonded14 two-body force
        for iatm, jatm, f_ij in bonded14_tbfs.items():
            gname_i = self.__iatm_to_groups[iatm]
            gname_j = self.__iatm_to_groups[jatm]

            r_ij = crd[iatm] - crd[jatm]
            # f_ij_r_ij = utility.tensor1(f_ij, r_ij)
            flux_pot[gname_i, gname_j] += f_ij_r_ij / volume_ij
            flux_pot[gname_j, gname_i] += f_ij_r_ij / volume_ji

        return flux_pot

    def _cal_nonbond_flux(self):

        # initialize
        group_names = self.get_groupnames()
        flux_pot = {}
        for gname_i in group_names:
            for gname_j in group_names:
                ts = numpy.zeros([3,3])
                flux_pot[gname_i, gname_j] = ts

        for iatm, jatm_ptr, f_ijs in gen_tbforces_each_atom:

            for j, f_ij in enumerate(f_ijs):
                jatm = jatm_ptr + j

                gname_i = self.__iatm_to_groups[iatm]
                gname_j = self.__iatm_to_groups[jatm]

                crd_ij = crd[iatm] - crd[jatm]
                # f_ij_r_ij = utility.tensor1(f_ij, crd_ij)
                flux_pot[gname_i, gname_j] += f_ij_r_ij / volume_ij
                flux_pot[gname_j, gname_i] += f_ij_r_ij / volume_ji

        return flux_pot



if __name__ == '__main__':
    cal = EnergyFluxCalculator()



#! /usr/bin/env python
from __future__ import print_function
import sys
import math
import numpy


class TensorParser:

    section_end = '%end'

    def __init__(self, filename):
        self.__filename = filename

    def parse(self):
        with open(self.__filename, 'rb') as file:
            names, tensors = self.parse_content(file)
        return names, tensors

    def parse_content(self, file):

        tensors = []

        gen_lines = self.gen_optimized_lines(file)
        sections = []
        for line in gen_lines:
            if not self.is_section(line): continue

            flagname = line[1:].strip()
            if flagname == 'data':
                lines = list( self.gen_section_lines(gen_lines) )
                names, tensor = self.parse_data(lines)
                tensors.append( tensor )

        return names, tensors

    def parse_data(self, lines):
        names = []
        tensors = []
        for line in lines:
            cols = line.split()
            names.append(cols[0])
            v = [float(c) for c in cols[1:]]
            tensor = [ [v[0], v[1], v[2]],
                       [v[3], v[4], v[5]],
                       [v[6], v[7], v[8]] ]
            tensors.append(tensor)

        # array = numpy.array(v)
        # print(array.reshape(3,3,len(array)))
        return names, numpy.array(tensors)

    def gen_optimized_lines(self, file):
        # first flag
        for line in file:
            if self.is_section(line):
                yield line
                break
            yield line

        for line in file:
            if self.is_section(line):
                yield self.section_end
                yield line
            yield line

        else:
            yield self.section_end

    def split_content(self, gen):
        lines = []
        for line in gen:
            if line.startswith('%'):
                pass

    def is_section(self, line):
        if line == self.section_end:
            return False
        elif line.startswith('%'):
            return True
        else:
            return False

    def gen_section_lines(self, gen):
        for line in gen:
            if line == self.section_end:
                break
            elif self.is_section(line):
                continue
            yield line


def split_data(data, interval=100):

    splitted_data = []
    num_data = len(data)
    num_intvls = num_data/interval

    for nint in range(num_intvls):
        split_data.append( data[nint*interval:(nint+1)*interval] )

    # The remains of data are thrown away.
    return splitted_data

def cal_average_with_scalar(values_traj):

    # natom times ntraj
    natom = len(values_traj[0])
    ntraj = len(values_traj)

    sum_values = numpy.zeros([natom])
    for values in values_traj:
        sum_values += values

    return sum_values / ntraj

def cal_average_with_tensor(tensors):

    sum_tensor = numpy.zeros(tensors[0].shape)

    for t in tensors:
        sum_tensor += t

    nten = len(tensors)
    return sum_tensor/nten

def cal_rmsf(tensors, average_tensor):

    rmsf_tensor = numpy.zeros(average_tensor.shape)

    for t in tensors:
        diff_ten = average_tensor - t
        rmsf_tensor += diff_ten * diff_ten

    nten = len(tensors)

    return numpy.sqrt(rmsf_tensor) / nten

def gen_eigens(tensors):
    """Generate the calculated eigen values for each tonsor."""
    # diagonal
    for tensor in tensors:
        eigen, vec = numpy.linalg.eig(tensor)
        try:
            val = math.sqrt(eigen[0]**2 + eigen[1]**2 + eigen[2]**2)
            yield val
        except ValueError:
            print(eigen)

def get_average_after_eigen(tensors_traj):
    evalues = []
    for itraj, tensors in enumerate(tensors_traj):
        evalues.append( numpy.array(list(gen_eigens(tensors))) )
    return cal_average_with_scalar(evalues)

def get_eigen_after_average(tensors_traj):
    average_tensors = cal_average_with_tensor(tensors_traj)
    return list(gen_eigens(average_tensors))

def simplify_tensor(filename, fns, labels='', snapshot=False, **kwds):
    """Show and make figure for stress ratio."""

    if len(fns) == 0:
        pass

    elif len(labels.split(',')) != len(fns):
        print(labels.split(','))
        print(fns)
        print("The number of labels and filenames must be same.")
        quit()

    # for total stress tensor
    tot_parser = TensorParser(filename)
    names, tot_data = tot_parser.parse()
    if snapshot:
        avetot_evalues = get_average_after_eigen(tot_data)
    else:
        avetot_evalues = get_eigen_after_average(tot_data)

    # for the stress tensor for each component
    evalues_list = []
    labels = [label.strip() for label in labels.split(',') ]
    for label, fn in zip(labels, fns):
        parser = TensorParser(fn)
        names, data = parser.parse()
        if snapshot:
            evalues_list.append( get_average_after_eigen(data) )
        else:
            evalues_list.append( get_eigen_after_average(data) )

    # output the eigen value of stress tensor
    if len(evalues_list) == 0:
        header = '{id:9s}  {name:>4s} {total:>12s}'.format(
                id='#label id', name='name', total='total')
        print(header)
        fmt = '{id:>9d}  {name:<4s} {total:12.7f}'

        for name, tot_evalue in zip(names, avetot_evalues):

            # name
            atom_id, aname = name.split('_')

            # output one line data
            print(fmt.format(id=int(atom_id), name=aname, total=tot_evalue))

    else:
        header = '{id:9s}  {name:>4s} {total:>12s}'.format(
                id='#label id', name='name', total='total') + ' '.join(
                        '{:>10s}'.format(label) for label in labels)
        print(header)
        fmt = '{id:>9}  {name:<4s} {total:12.7f}' + ' '.join(
                '{'+label+':10.2f}' for label in labels)

        for tuples in zip(names, avetot_evalues, *evalues_list):

            name, tot, others = tuples[0], tuples[1], tuples[2:]

            # name
            atom_id, aname = name.split('_')

            # each evalue
            if not isinstance(others, tuple):
                others = [others]
            label_to_evalues = dict(zip(labels, others))

            # output one line data
            print(fmt.format(id=atom_id, name=aname,
                total=tot, **label_to_evalues))


if __name__ == '__main__':
    from wurp.console import arg_simplify, exec_command

    parser = arg_simplify()
    exec_command(parser)



from __future__ import print_function

import os, sys
from collections import OrderedDict as odict

topdir = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
if topdir not in sys.path:
    sys.path.insert(0, topdir)
import exception

class IniParser:

    """
    $ cat target_file
    [SEC_1]
    attr1 = val1
    attr2 = val2
    1 2 3 4
    [SEC_2]
    5 6 7 8 9 10
    .
    .
    .
    [SEC_N]
    attr1 = val1
    attr2 = val2
    100 101 102

    =>

    {'SEC1':[1,2,3,4], 'SEC_2' [5,6,7,8,9,10], ..., 'SEC_N': [100,101,102]}
    """

    section_end = "#[END_SECTION]"
    comment = '#'

    def __init__(self, filename):
        self.__filename = filename
        self.parse()

    def __getitem__(self, secname):
        return self.get_vals(secname)

    def get_filename(self):
        return self.__filename
        
    # def __setitem__(self, secname, value):
    #     pass

    def get_secnames(self):
        """Get the section names in order."""
        return self.__secname_to_vals.keys()

    def get_vals(self, secname):
        """Get values for section name."""
        return self.__secname_to_vals[secname]

    def get_attr_dict(self, secname):
        """Get attribute value dictionary for section name."""
        return self.__secname_to_attrdict[secname]

    def parse(self):
        """Parse ini format content."""

        with open(self.__filename, 'r') as file:
            secname_lines_pairs = self._parse_secname_lines_pairs(file)

        # parse vals
        self.__secname_to_vals = odict()
        for secname, lines in secname_lines_pairs:
            self.__secname_to_vals[secname] = list(self._gen_col(lines))

        # parse attributes
        self.__secname_to_attrdict = odict()
        for secname, lines, in secname_lines_pairs:
            self.__secname_to_attrdict[secname] = odict(
                    self._gen_attr_val(lines))

    def gen_secname_cols_pair(self):
        """Generate section_name: columns pair."""
        for secname in self.get_secnames():
            yield secname, self[secname]

    def _gen_col(self, lines):
        for line in lines:
            if '=' in line: continue
            for col in line.split():
                yield col

    def _gen_attr_val(self, lines):
        for line in lines:
            cols = line.split('=')
            if len(cols) == 2:
                yield cols[0].strip(), cols[1].strip()

    def _parse_secname_lines_pairs(self, file):
        sections = []
        gen_lines = self._gen_optimized_lines(file)

        for line in gen_lines:
            if self._is_section(line):
                lines = list(self._gen_section_lines(gen_lines))
                sections.append( (line[1:-1].strip(), lines) )

        return sections

    def _gen_section_lines(self, gen):
        for line in gen:
            if line == self.section_end:
                break
            yield line

    def _gen_optimized_lines(self, file):
        """Generate lines with the end of sections and without the comments."""

        # search first section marker
        for line in file:
            line = line.split(self.comment)[0].strip()
            if self._is_section(line):
                yield line
                break
            elif line == '':
                continue
            else:
                mes = 'found the invalid string before first section marker: '
                raise ParserError(mes + line)

        # rest
        for line in file:
            line = line.split(self.comment)[0].strip()
            if self._is_section(line):
                yield self.section_end
                yield line
            elif line != '':
                yield line
            else:
                continue
        else:
            yield self.section_end

    def _is_section(self, line):
        if line == self.section_end:
            return False
        elif line.startswith('[') and line.endswith(']'):
            return True
        elif line.startswith('[') or line.endswith(']'):
            raise SectionNotFoundError(line)
        else:
            return False





from __future__ import print_function

scale_factor = 20.455 * 10.0**(-3)

try:
    import lib_parser
except ImportError:
    lib_parser = None

from exception import WurpException
class NumAtomInvalidError(WurpException): pass


import gzip
import numpy

class ParserPyBase:

    def __init__(self, filename, natom, use_pbc=False):
        self.__natom = natom
        self.__use_pbc = use_pbc

        if filename.endswith('.gz'):
            self.__file = gzip.open(filename, 'rb')
        else:
            self.__file = open(filename, 'rb')
        title = self.parse_header(self.__file)

    def _set_module(self, module):
        pass

    def __next__(self):
        crdvel = list(self.parse_crdvel_iter(self.__file, self.__natom))
        box = self.parse_pbc(self.__file) if self.__use_pbc else None
        if len(crdvel) == 0:
            raise StopIteration()

        return numpy.array(crdvel), box

    next = __next__ # for python 2.x

    def __iter__(self):
        return self

    def parse_header(self, file):
        return file.next()

    def parse_data(self, file, natom):
        nline = natom*3 / 10
        rem   = natom*3 % 10
        try:
            for iline in range(nline):
                line = file.next()
                for icol in range(10):
                    beg, end = 8*icol, 8*(icol+1)
                    yield float(line[beg:end])

            else:
                if rem != 0:
                    line = file.next()
                    for icol in range(rem):
                        beg, end = 8*icol, 8*(icol+1)
                        yield float(line[beg:end])

        except ValueError:
            msg = "Given the number of atoms is {}.".format(natom)
            raise NumAtomInvalidError(msg)

    def parse_crdvel_iter(self, file, natom):
        gen_data = self.parse_data(file, natom)

        for iatm in range(natom):
            x = gen_data.next()
            y = gen_data.next()
            z = gen_data.next()
            yield x, y, z

    def parse_pbc(self, file):

        # parse the info of periodic boudary condition
        line = file.next()
        p1, p2, p3 = line[0:8],  line[8:16], line[16:24]
        return [float(p) for p in [p1, p2, p3]]

    def close(self):
        self.__file.close()

    def get_dt(self):
        return 0.1


class ParserFortBase:

    def __init__(self, filename, natom, use_pbc=False):
        self.__mod.initialize(filename)
        self.__natom = natom
        self.__use_pbc = use_pbc

    def _set_module(self, module):
        self.__mod = module

    def __next__(self):
        crd, ierr = self.__mod.parse_next(self.__natom)
        if ierr != 0:
            self.__mod.finalize()
            raise StopIteration()

        if self.__use_pbc:
            box, ierr = self.__mod.parse_pbc(self.__natom)
            return crd, box
        else:
            return crd, None
    next = __next__ # for python 2.x

    def __iter__(self):
        return self

ParserBase = ParserPyBase

class CoordinateParser(ParserBase):

    def __init__(self, *args, **kwds):
        if lib_parser:
            self._set_module(lib_parser.formatted_coordinate)
        ParserBase.__init__(self, *args, **kwds)

class VelocityParser_half_dt(ParserBase):

    def __init__(self, *args, **kwds):
        if lib_parser:
            self._set_module(lib_parser.formatted_velocity)
        ParserBase.__init__(self, *args, **kwds)

        self.__vel_old, box = ParserBase.__next__(self)

    def __next__(self):
        vel, box = ParserBase.__next__(self)
        vel_half = 0.5 * (vel + self.__vel_old)
        self.__vel_old = vel
        return scale_factor*vel_half, box

    next = __next__ # for python 2.x

class VelocityParserNormal(ParserBase):

    def __init__(self, *args, **kwds):
        if lib_parser:
            self._set_module(lib_parser.formatted_velocity)
        kwds['use_pbc'] = False
        ParserBase.__init__(self, *args, **kwds)

    def __next__(self):
        vel, box = ParserBase.__next__(self)
        return scale_factor * vel, box

    next = __next__ # for python 2.x

VelocityParser = VelocityParserNormal

import netCDF4 as netcdf

class NetCDFReaderBase:

    trjname = None # A inheritanced class must have this variable.
    scale_factor   = 1.0

    def __init__(self, filename, natom=None, use_pbc=False):

        self.__fp = filename

        # read NetCDF file
        try:
            from netCDF4 import Dataset
        except ImportError:
            raise
        trjobj = Dataset(filename, mode='r')

        self._natom  = len(trjobj.dimensions['atom'])
        self._nframe = len(trjobj.dimensions['frame'])
        self.title   = getattr(trjobj, 'title', None)
        if use_pbc:
            if 'cell_lengths' in trjobj.variables: #add 17.06.29
                self._box1 = trjobj.variables['cell_lengths']
            if 'cell_angles' in trjobj.variables:  #add 17.06.29
                self._box2 = trjobj.variables['cell_angles']
        self.__use_pbc = use_pbc

        self.__iter_cnt = 0

        trjobj.sync()
        trjobj.close()
        self.__trjobj = None

    def __getitem__(self, ifrm):
        trjobj = self.open()
        trj_vars = trjobj.variables
        time = trj_vars['time'][ifrm]
        crdvel = trj_vars[self.trjname][ifrm]

        if self.__use_pbc:
            if 'cell_lengths' in trj_vars: #add 17.06.29
                box1 = trj_vars['cell_lengths'][ifrm]
            else:  #add 17.06.29
                box1=''  #add 17.06.29
            if 'cell_angles' in trj_vars:  #add 17.06.29
                box2 = trj_vars['cell_angles'][ifrm]
            else:  #add 17.06.29
                box2=''  #add 17.06.29
        else:
            n = len(crdvel)
            box1, box2 = None, None

        self.close()

        #TODO convert the box information 1 matrix.
        box = box1

        return crdvel*self.scale_factor, box

    def __next__(self):
        try:
            crdvel, box = self[self.__iter_cnt]
            self.__iter_cnt += 1

        except IndexError:
            self.__iter_cnt = 0
            raise StopIteration()

        return crdvel, box

    next = __next__ # for python 2.x
    def __iter__(self): return self

    def parse_header(self, file):
        return file.next()

    def open(self):
        if self.__trjobj is None:
            trjobj = netcdf.Dataset(self.__fp, mode='r')
            self.__trjobj = trjobj

        return self.__trjobj

    def close(self):
        if self.__trjobj:
            self.__trjobj.sync()
            self.__trjobj.close()
            self.__trjobj = None

    def get_dt(self):
        trjobj = self.open()
        trj_vars = trjobj.variables
        times = trj_vars['time'][ifrm]
        dt = times[1] - times[0]
        self.close()
        return dt

class NetCDFReaderBaseNew:

    trjname = None # A inheritanced class must have this variable.
    scale_factor   = 1.0

    def __init__(self, filename, natom=None, use_pbc=False):

        self.__filename = filename
        trjobj = self.open(filename)

        self._natom  = len(trjobj.dimensions['atom'])
        self._nframe = len(trjobj.dimensions['frame'])
        self.title   = getattr(trjobj, 'title', None)
        if use_pbc:
            self._box1 = trjobj.variables['cell_lengths']
            self._box2 = trjobj.variables['cell_angles']
        self.__use_pbc = use_pbc

        self.__iter_cnt = 0

    def open(self, filename):
        # read NetCDF file
        try:
            from netCDF4 import Dataset
        except ImportError:
            raise
        trjobj = Dataset(filename)
        return trjobj


    def __getitem__(self, ifrm):
        trjobj = self.open(self.__filename)
        trj_vars = trjobj.variables
        time = trj_vars['time'][ifrm]
        crdvel = trj_vars[self.trjname][ifrm]

        if self.__use_pbc:
            box1 = trj_vars['cell_lengths'][ifrm]
            box2 = trj_vars['cell_angles'][ifrm]
        else:
            n = len(crdvel)
            box1, box2 = None, None

        trjobj.close()

        return crdvel*self.scale_factor, (box1, box2)

    def __next__(self):
        try:
            crdvel, box = self[self.__iter_cnt]
            self.__iter_cnt += 1

        except IndexError:
            self.__iter_cnt = 0
            raise StopIteration()

        return crdvel, box

    next = __next__ # for python 2.x
    def __iter__(self): return self

    def parse_header(self, file):
        return file.next()

    def close(self):
        pass
        # self.__trjobj.close()


class NetCDFCoordinateReader(NetCDFReaderBase):

    trjname = 'coordinates'

    def __init__(self, *args, **kwds):
        NetCDFReaderBase.__init__(self, *args, **kwds)


class NetCDFVelocityReader(NetCDFReaderBase):

    trjname = 'velocities'
    # The velocity unit in NetCDF is angstrom/(ps/20.455)
    scale_factor = 1.0/10.0**3  # A/ps <= A/fs

    def __init__(self, *args, **kwds):
        NetCDFReaderBase.__init__(self, *args, **kwds)


class InvalidAtomNumber: pass
class RestartParser:

    def __init__(self, filename, natom, use_pbc=False, trj_type='crdvel'):
        self.__filename = filename
        self.__natom    = natom
        self.__use_pbc  = use_pbc

    @classmethod
    def set_trjtype(cls, trj_type='crd'):

        cls.__use_crd = False
        cls.__use_vel = False
        cls.__parsed = False

        if trj_type == 'crd':
            cls.__use_crd = True
        elif trj_type == 'vel':
            cls.__use_vel = True
        else:
            pass

    def __next__(self):

        if self.__parsed:
            raise StopIteration()

        crd, vel, pbc = self.parse()

        if self.__use_crd:
            self.__parsed = True
            return crd, pbc
        elif self.__use_vel:
            self.__parsed = True
            return vel, pbc
        else: 
            self.__parsed = True
            return (crd, vel), pbc

    next = __next__ # for python 2.x

    def __iter__(self):
        return self

    def parse(self):
        with open(self.__filename, 'rb') as file:
            title, natom, simtime = self.parse_header(file)
            import numpy
            crd = list( self.parse_crd_iter(file, natom) )
            vel = list( self.parse_vel_iter(file, natom) )

            if self.__use_pbc:
                pbc = self.parse_pbc(file)
            else:
                pbc = None

        return numpy.array(crd), numpy.array(vel), pbc

    def close(self):
        pass

    def parse_header(self, file):
        # parse title line
        title = file.next()

        # parse info line
        line = file.next()
        cols = line.split()
        natom, simtime = int(cols[0]), float(cols[1])

        if natom != self.__natom:
            raise InvalidAtomNumber()

        return title, natom, simtime

    def parse_data(self, file, natom):
        for iline in range(natom/2):
            line = file.next()
            for icol in range(6):
                beg, end = 12*icol, 12*(icol+1)
                yield float(line[beg:end])

        else:
            if natom%2 == 1:
                line = file.next()
                for icol in range(3):
                    beg, end = 12*icol, 12*(icol+1)
                    yield float(line[beg:end])

    def parse_crd_iter(self, file, natom):
        gen_data = self.parse_data(file, natom)

        for iatm in range(natom):
            x = gen_data.next()
            y = gen_data.next()
            z = gen_data.next()
            yield x, y, z

    def parse_vel_iter(self, file, natom):
        gen_data = self.parse_data(file, natom)
        c = scale_factor

        for iatm in range(natom):
            x = gen_data.next() * c
            y = gen_data.next() * c
            z = gen_data.next() * c
            yield x, y, z

    def parse_pbc(self, file):

        # parse the info of periodic boudary condition
        try:
            line = file.next()
            p1, p2, p3 = line[0:12],  line[12:24], line[24:36]
            p4, p5, p6 = line[36:48], line[48:60], line[60:72]
            pbc = [float(p) for p in [p1, p2, p3, p4, p5, p6]]
        except StopIteration:
            pbc = None

        return None


class TrajectoryWriterBase:

    suffix = ''

    def __init__(self, filepath, fst_lst_int=(1,-1,1),
            use_pbc=False,rotate=1,compress=0):
        self.__fp = filepath
        self.__rotate = rotate
        self.__fst_lst_int = fst_lst_int
        self.__compress = compress
        self.__use_pbc = use_pbc

    def write_header(self):
        pass

    def write_all(self, trj):
        for snap in trj:
            self.write(snap)

        self.close()

    def open(self):
        fp = self.get_fp()
        self.__file = open(fp, 'wb')
        return self.__file

    def write(self, ifrm, snap, time, box):
        pass

    def check(self):
        fp = self.get_fp()
        if os.path.exists(fp):
            raise TrajectoryExistsError(fp)

    def close(self):
        self.__file.close()

    def get_fp(self):
        return self.__fp

    def use_pbc(self):
        return self.__use_pbc

import lib_trj
class AsciiCoordinateWriter(TrajectoryWriterBase):

    suffix = 'mdcrd'

    def __init__(self, *args, **kwds):
        TrajectoryWriterBase.__init__(self, *args, **kwds)
        self.write_header()

    def write_header(self):
        msg = "This trajectory was generated by the CURP."
        file = self.open()
        file.write(msg+'\n')
        file.close()

    def write(self, ifrm, crd, time=None, box=None):
        fp = self.get_fp()

        # write next coordinate
        lib_trj.write_trajectory(crd, fp)


class AsciiVelocityWriter(TrajectoryWriterBase):

    suffix = 'mdvel'
    scale_factor = 10.0**3 / 20.455

    def __init__(self, *args, **kwds):
        TrajectoryWriterBase.__init__(self, *args, **kwds)
        self.write_header()

    def write_header(self):
        msg = "This trajectory was generated by the CURP."
        file = self.open()
        file.write(msg+'\n')
        file.close()

    def write(self, ifrm, vel, time=None, box=None):
        fp = self.get_fp()

        # write next velocity
        # convert wurp unit into amber unit for velocity
        c = self.scale_factor
        lib_trj.write_trajectory(vel*c, fp)


class NetCDFWriterBase(TrajectoryWriterBase):

    # Please set these class variables in a inheritant class.
    suffix   = 'xxx.nc'
    trj_type = ''
    units    = ''
    ncdf_scale = 1.0
    scale_factor = 1.0

    # class variable
    version  = 1.0

    def __init__(self, *args, **kwds):
        self.__ncfile = None
        self.__ifrm   = 0
        TrajectoryWriterBase.__init__(self, *args, **kwds)

    def setup(self, natom):
        """Setup for NetCDF dataset."""

        ncfile = netcdf.Dataset(self.get_fp(), clobber=True,
                mode='w', format='NETCDF3_64BIT')

        # set global attributes
        ncfile.title             = 'netCDF '+self.trj_type +'trajectory'
        ncfile.application       = 'CURP'
        ncfile.program           = 'Utility tools in the CURP'
        ncfile.programVersion    = '0.6'
        ncfile.Convetsions       = 'AMBER'
        ncfile.ConvetsionVersion = str(self.version)

        # create dimensions
        ncfile.createDimension('frame', None)
        ncfile.createDimension('spatial', 3)
        ncfile.createDimension('atom', natom)
        ncfile.createDimension('cell_spatial', 3)
        ncfile.createDimension('cell_angular', 3)
        # file.createDimensions('label', ?)

        # create variables
        time = ncfile.createVariable('time', 'f4', ('frame',))
        time.units = 'picosecond'

        trj = ncfile.createVariable(self.trj_type, # coordinates or velocities
                'f4', ('frame', 'atom', 'spatial' ))
        trj.units = self.units
        trj.scale_factor = self.ncdf_scale


        cell_lengths = ncfile.createVariable('cell_lengths',
                'f8', ('frame', 'cell_spatial'))
        cell_lengths.units = 'angstrom'
        
        cell_angles = ncfile.createVariable('cell_angles',
                'f8', ('frame', 'cell_angular'))
        cell_angles.units = 'degree'

        # file.createVariable('spatial', 'c')
        # file.variables['spatial'] = 'xyz'
        # print(file.variables)

        # sync
        ncfile.sync()
        ncfile.close()

    def write_header(self):
        """Write header."""
        pass

    def write(self, ifrm, snap, time=None, box=None):
        """Write snapshots to NetCDF file object."""

        # setup at first step
        if self.__ifrm == 0:
            natom = len(snap)
            self.setup(natom)

        ncfile = self.open()
        # ncfile = self.__ncfile

        self.__ifrm += 1
        ifrm_1 = self.__ifrm - 1

        nc_trj = ncfile.variables[self.trj_type]
        nc_time = ncfile.variables['time']

        nc_trj[ifrm_1] = snap[:]/self.scale_factor
        nc_time[ifrm_1] = time

        if isinstance(box,numpy.ndarray): # periodic boundary conditios ##add 17.06.29
            print('here', box)
            ncfile.variables['cell_lengths'][ifrm_1] = box
            ncfile.variables['cell_angles'][ifrm_1]  = box
            # make this code right in next version.

        self.close()

    def open(self):
        if self.__ncfile is None:
            ncfile = netcdf.Dataset(self.get_fp(), clobber=True, mode='r+')
            self.__ncfile = ncfile

        return self.__ncfile

    def close(self):
        if self.__ncfile:
            self.__ncfile.sync()
            self.__ncfile.close()
            self.__ncfile = None
        

class NetCDFCoordinateWriter(NetCDFWriterBase):

    suffix   = 'crd.nc'
    trj_type = 'coordinates'
    units    = 'angstrom'

    def __init__(self, *args, **kwds):
        NetCDFWriterBase.__init__(self, *args, **kwds)


class NetCDFVelocityWriter(NetCDFWriterBase):

    suffix   = 'vel.nc'
    trj_type = 'velocities'
    units    = 'angstrom/picosecond'
    ncdf_scale = 20.455
    scale_factor = 1.0/10.0**3 # fs => ps

    def __init__(self, *args, **kwds):
        NetCDFWriterBase.__init__(self, *args, **kwds)


if __name__ == "__main__":

    import os

    wurp_path = os.environ['CURP_HOME']
    exapmle_path = prmtop_fn = os.path.join(
            wurp_path,'test','amber-enk-vacuum')

    prmtop_fn = os.path.join(exapmle_path, 'prmtop_fn')
    crds_fn = os.path.join(exapmle_path, 'sam.nccrd')
    vels_fn = os.path.join(exapmle_path, 'sam.ncvel')

    from benchmarker import Benchmarker
    with Benchmarker(width=20) as bm:
        # with bm('parse'):
            # rst_fn = "test/system.rst"
            # parser = RestartParser(rst_fn, 30661)
            # crd, vel, pbc = parser.parse()
            # crd, vel = parser.parse()
            # print(vel)
        # with bm('print'):
        #     crd, vel
        #     print()
        #     for p in crd:
        #         print(p)
        #     for p in vel:
        #         print(p)

        # print('crd')
        # for c in crd:
        #     print(c)
        # print('vel')
        # for v in vel:
        #     print(v)
        # print('pbc')
        # print(pbc)

        # with bm('crd'):
        #     print()
        #     parser = CoordinateParser('test/sam-nwat.mdcrd', 1799)
        #     i = 0
        #     for crd, box in parser:
        #         i += 1
        #         if i%100==0:
        #             print('{:>5} x 10^2: size = {:>5}'.format(
        #                 i/100, len(crd)))
        #     print(i)
                    
        # with bm('crd+pbc'):
            # print()
            # parser = CoordinateParser('test/ala3-woct.mdcrd.gz',
                    # 5844, use_pbc=True)
            # i = 0
            # for crd, box in parser:
                # i += 1
                # if i%100==0:
                    # print('{:>5} x 10^2: size = {:>5}'.format(
                        # i/100, len(crd)))

            # print(i)

        with bm('NetCDF'):
            crd_trj = NetCDFCoordinateReader(crds_fn)
            for crd, pbc in crd_trj:
                crd
                # print(crd)

            vel_trj = NetCDFVelocityReader(vels_fn)
            for vel, pbc in vel_trj:
                vel
                # print(vel/scale_factor) # A/fs => A/(ps/20.455)





# import numpy
# class DimensionError(Exception): pass
# class RangeError(Exception): pass
# class MultiArray(dict):

#     def __init__(self, dim=2, default=0.0):
#         self.__ndim     = dim
#         self.__default = default

#     def __getitem__(self, key):
#         self._check_key(key)
#         return self.get(key, self.__default)

#     def __setitem__(self, key, value):
#         self._check_key(key)
#         super(self.__class__, self).__setitem__(key, value)

#     def _check_key(self, key):
#         if len(key) != self.__ndim:
#             raise DimensionError(key)
#         for k in key:
#             if not isinstance(k, int):
#                 raise TypeError(k)
#             if k < 0:
#                 raise RangeError(k)

#     def array(self):
#         dims = zip(*self.keys())
#         dim_maxs = [max(dim) for dim in dims]
#         # dim_mins = [min(dim) for dim in dims]

#         array = numpy.zeros(dim_maxs, dtype=float)
#         for dmax in dim_maxs:

import numpy

def tensor(xs, ys):
    axes_x = xs.shape
    axes_y = ys.shape 

    if axes_x != axes_y:
        print(axes_x, axes_y)
        raise ValueError

    dim0 = axes_x[-1]

    if len(axes_x) == 1:
        result = numpy.zeros([dim0, dim0])
    elif len(axes_x) == 2:
        result = numpy.zeros([axes_x[0], dim0, dim0])
    elif len(axes_x) == 3:
        result = numpy.zeros([axes_x[0], axes_x[1], dim0, dim0])
    elif len(axes_x) == 4:
        result = numpy.zeros([axes_x[0], axes_x[1], axes_x[2], dim0, dim0])
    else:
        raise numpy.DimensionError()

    for i in range(dim0):
        for j in range(dim0):
            result[..., i, j] = xs[..., i] * ys[..., j]

    return result

def tensor1(x, y):
    result = numpy.zeros([3,3])
    for i in range(3):
        for j in range(3):
            result[i, j] = x[i] * y[j]

    return result

def tensor2(xs, ys):
    axes_x = xs.shape
    axes_y = ys.shape 

    if axes_x != axes_y:
        raise ValueError

    dim0 = axes_x[-1]
    if len(axes_x) == 2:
        result = numpy.zeros([axes_x[0], dim0, dim0])
        for n, (x, y) in enumerate(zip(xs,ys)):
            for i in range(dim0):
                for j in range(dim0):
                    result[n, i, j] = x[i] * y[j]

        return result


def time_tensor(natom=10**5):
    xs = numpy.ones([natom,3])
    ys = numpy.ones([natom,3])

    from benchmarker import Benchmarker
    with Benchmarker(width=20) as bm:

        with bm('tensor'):
            result = tensor2(xs, ys)

        with bm('tensor 1'):
            result = numpy.zeros([natom,3,3])
            for i,(x, y) in enumerate(zip(xs, ys)):
                result[i] = tensor1(x,y)
            # print(result)

        with bm('tensor2'):
            result = tensor3(xs, ys)

def time_tensor2(natom=10**5):
    xs = numpy.ones([100, natom,3])
    ys = numpy.ones([100, natom,3])
    # xs = numpy.array([1,2,3])
    # ys = numpy.array([4,5,6])

    from benchmarker import Benchmarker
    with Benchmarker(width=10) as bm:
        with bm('tensor'):
            result = tensor(xs, ys)

        with bm('dot product'):
            aa = numpy.arange(10**5)
            bb = numpy.arange(10**5)
            aabb = numpy.dot(aa, bb)
        print(result)
        print(aabb)

# def tensor(xs, ys):
#     result = numpy.zeros(xs.shape)
#     for x, y in zip(xs, ys):
#          = x[0]*y[0], x[0]*y[1], x[0]*y[2]

# from scipy import sparse
class InvalidShapeError: pass
class TBFMatrix_Old:
    """
    The old class for two-body force
    """

    def __init__(self, shape):
        self.__shape = shape
        self.xs = sparse.lil_matrix(shape)
        self.ys = sparse.lil_matrix(shape)
        self.zs = sparse.lil_matrix(shape)

    def get_shape(self):
        return self.__shape

    def __setitem__(self, index, value):
        self.xs[index] = value[0]
        self.ys[index] = value[1]
        self.zs[index] = value[2]

    def __getitem__(self, index):
        return numpy.array([self.xs[index],self.ys[index],self.zs[index]])

    def __add__(self, other):
        if self.__shape != other.get_shape():
            raise InvalidShapeError
        new = TBFMatrix(self.__shape)
        new.xs = self.xs + other.xs
        new.ys = self.ys + other.ys
        new.zs = self.zs + other.zs
        return new

    def iadd(self, index, value):
        self.xs[index] += value[0]
        self.ys[index] += value[1]
        self.zs[index] += value[2]

    def isub(self, index, value):
        self.xs[index] -= value[0]
        self.ys[index] -= value[1]
        self.zs[index] -= value[2]

    def __iadd__(self, other):
        if self.__shape != other.get_shape():
            raise InvalidShapeError
        self.xs += other.xs
        self.ys += other.ys
        self.zs += other.zs
        return self

    def minus(self, value):
        self.xs[index] -= value[0]
        self.ys[index] -= value[1]
        self.zs[index] -= value[2]

    def __isub__(self, value):
        self.xs[index] -= value[0]
        self.ys[index] -= value[1]
        self.zs[index] -= value[2]

    def items(self):
        ilist, jlist = self.xs.nonzero()
        for i, j in zip(ilist, jlist):
            yield i, j, numpy.array([
                self.xs[i,j], self.ys[i,j], self.zs[i,j] ])

    def __repr__(self):
        mes = ( '[{i},{j}] = {value}'.format(i=i,j=j,value=str(ary))
                for i, j, ary in self.items() )

        return '\n'.join(mes)


class TBFMatrix(dict):

    def __init__(self, shape):
        dict.__init__(self)
        self.__shape = shape

    def get_shape(self):
        return self.__shape

    def __getitem__(self, key):
        return self.get(key, numpy.zeros([3]))

    def __add__(self, other):
        if self.__shape != other.get_shape():
            raise InvalidShapeError
        new = self.copy()
        for key, value in other.items():
            new[key] += value
        return new

    def tadd(self, key, value):
        i, j = key
        self[i, j] += value
        self[j, i] -= value


import functools
def timeit(fun):
    import time
    @functools.wraps(fun)
    def wrapped(*args, **kwargs):
        t0 = time.time()
        result = fun(*args, **kwargs)
        t1 = time.time()
        print("{0} : {1}").format(fn.__name__, t1 - t0)
        return result
    return wrapped

class TimeStore:

    def __init__(self):
        self.__tag_to_times = {}
        self.__tags = []

    def store_time(self, tag, time):
        if tag not in self.__tags:
            self.__tag_to_times[tag] = []
            self.__tags.append(tag)

        self.__tag_to_times[tag].append(time)

    def gen_time_info(self):
        for tag in self.__tags:
            yield tag, self.__tag_to_times[tag]


if __name__ == '__main__':
    time_tensor2()



#! /usr/bin/env python2
from __future__ import print_function
import sys, os

class BWDataParser:

    def __init__(self, filename):
        self.parse(filename)

    def parse(self, filename):
        file = open(filename, 'rb')
        self.rid_to_name_bw_pair = {}

        lines = ( line.strip() for line in file
                 if line.strip()
                 if not line.strip().startswith('%') 
                 if not line.strip().startswith('#') )


        for line in lines:

            rname, rid, bw_id = line.split()
            rid = int(rid)
            self.rid_to_name_bw_pair[rid] = (rname, bw_id)

        file.close()

if __name__ == '__main__':

    ec_file  = sys.stdin
    fn = sys.argv[1]

    parser = BWDataParser(fn)
    rid_to_name_bw_pair = parser.rid_to_name_bw_pair

    for line in ec_file:
        line = line.strip()
        if not line: continue

        if line.startswith('#'):
            continue

        cols = line.split()
        don_line, acc_line, rest = cols[0], cols[1], cols[2:]

        rid1 = don_line.split('_')[0]
        rid2 = acc_line.split('_')[0]

        if int(rid1) in rid_to_name_bw_pair:
            name1, bwid1 = rid_to_name_bw_pair.get(int(rid1))
            don_line = '{}_{}{} '.format(rid1, name1, bwid1)

        if int(rid2) in rid_to_name_bw_pair:
            name2, bwid2 = rid_to_name_bw_pair.get(int(rid2))
            acc_line = '{}_{}{} '.format(rid2, name2, bwid2)

        print(don_line, acc_line, '  '.join(rest))




import sys
import numpy

section_end = '%end'

def parse_content(file):

    tensors = []

    gen_lines = gen_optimized_lines(file)
    sections = []
    for line in gen_lines:
        if not is_section(line): continue

        flagname = line[1:].strip()
        if flagname == 'data':
            lines = list( gen_section_lines(gen_lines) )
            names, tensor = parse_data(lines)
            tensors.append( numpy.array(tensor) )

    return names, tensors

def parse_data(lines):
    names = []
    tensor = []
    for line in lines:
        cols = line.split()
        names.append(cols[0])
        tensor.append( [float(c) for c in cols[1:]] )

    return names, tensor

def gen_optimized_lines(file):
    # first flag
    for line in file:
        if is_section(line):
            yield line
            break
        yield line

    for line in file:
        if is_section(line):
            yield section_end
            yield line
        yield line

    else:
        yield section_end

def split_content(gen):
    lines = []
    for line in gen:
        if line.startswith('%'):
            pass

def is_section(line):
    if line == section_end:
        return False
    elif line.startswith('%'):
        return True
    else:
        return False

def gen_section_lines(gen):
    for line in gen:
        if line == section_end:
            break
        elif is_section(line):
            continue
        yield line



            


def cal_average(tensors):

    sum_tensor = numpy.zeros(tensors[0].shape)

    for t in tensors:
        sum_tensor += t

    nten = len(tensors)
    return sum_tensor/nten

def cal_rmsf(tensors, average_tensor):

    rmsf_tensor = numpy.zeros(average_tensor.shape)

    for t in tensors:
        diff_ten = average_tensor - t
        rmsf_tensor += diff_ten * diff_ten

    nten = len(tensors)

    return numpy.sqrt(rmsf_tensor) / nten

if __name__ == '__main__':

    # make argument parser
    import argparse
    parser = argparse.ArgumentParser('Get the averaged tensor.')

    # add argument definitions
    parser.add_argument(
            '-i', '--input-data', dest='data_filename', required=True,
            help='specify input filename for the stress data.')

    # make arguments
    args = parser.parse_args()

    filename = args.data_filename
    with open(filename, 'rb') as file:
        names, tensors = parse_content(file)

    ave_tensor = cal_average(tensors)
    rmsf_tensor = cal_rmsf(tensors, ave_tensor)
    for name, avet, rmsft in zip(names, ave_tensor, rmsf_tensor):
        print(name)
        print(avet)
        print(rmsft)

    





#  -*- encoding: utf-8 -*-
"""exception - Exception handler"""
import os, sys
import traceback

from datetime import datetime
from pprint import pformat

raise_exception = True

class WurpException(Exception):

    def __init__(self, data=None, log_fn='log.txt', tb=False):
        BaseException.__init__(self)
        self.message = 'Wurp Exception'
        self.log_fn = log_fn
        self.date = datetime.now()
        self.__data = data
        self.__traceback = tb

#    def __repr__(self):
#        return 'WurpException'

    def __str__(self):
        mes = pformat(self.__data) if self.__data else ''
        if self.__traceback:
            mes += '\n'
            mes += traceback.format_exc()

        return mes

    def log(self):
        print self.__data

    def logerror(self):
        logfile = open(self.log_fn, 'a')
        logfile.write(str(self.date) + ' : ' + str(self))
        logfile.close()

    def getMessage(self):
        return self.__data




from __future__ import print_function

import itertools

################################################################################
class InteractionTableGenerator:

    """

    data structure:
        [jint, (iatm, jatm_begin, jatm_end)]

    >>> table1 = InteractionTable(natom)
    >>> table1
    <InteractionTable>

    exclude the bonded pair
    >>> table2 = table1.filter(if_excluded_bond)
    <InteractionTable>

    >>> list(table2)

    perform cutoff with cutoff length to make a interaction table within
    length from each atoms
    >>> table3 = table2.cutoff(crd, 10.0)
    >>> table.map()
    """

    def __init__(self, natom=None, base_table=None):
        if natom:
            self.__base_generater = self._init_table(natom)
        else:
            if base_table:
                self.__base_generater = base_table
            else:
                self.__base_generater = []

        self.__filters = []
        self.__cache = None

    def _init_table(self, natom):
        for iatm_1 in range(natom-1):
            iatm = iatm_1 + 1
            yield (iatm, iatm+1, natom)

    def filter(self, if_fun):
        self.__filters.append(if_fun)
        return self 

    def insert_filter(self, if_fun):
        self.__filters.insert(0, if_fun)
        return self

    def list(self):
        return list(self.__apply())

    def __iter__(self):
        return self

    def next(self):
        return self.__apply().next()
    __next__ = next # version 3.x

    def copy(self):
        gen1, gen2 = itertools.tee(self.__apply())
        self.__apply = gen1
        self.__filters = []
        return InteractionTable(base_table=gen2)

    def __apply(self):
        """Generator: apply if_function to the table."""

        if self.__filters == []:
            for iatm, jatm_beg, jatm_end in self.__base_generater:
                yield iatm, jatm_beg, jatm_end

        for iatm, jatm_beg, jatm_end in self.__base_generater:

            jatm_beg_new = 0
            jatm_end_new = jatm_beg
            
            for jatm in range(jatm_beg, jatm_end+1):
                print('jatm before', jatm, jatm_end)

                for if_fun in self.__filters:
                    if not if_fun(iatm, jatm):
                        print('aaa')
                        if jatm_beg_new > 0:
                            print('bbb',iatm, jatm_beg_new, jatm_end_new)
                ########################### here bug ########################
                            yield iatm, jatm_beg_new, jatm_end_new
                            print('ccc')
                            jatm_beg_new = 0
                        print('ddd')
                        # break

                else: # filters are all ok.
                    print('filter else', jatm, jatm_end)
                    if jatm_beg_new==0: jatm_beg_new = jatm
                    jatm_end_new = jatm

                print('jatm after', jatm)

            else:
                print('jatm else', jatm)
                if jatm_beg_new > 0:
                    yield iatm, jatm_beg_new, jatm_end_new

    def cutoff(self, crd, length):
        return self.filter(self._if_cutoff(crd, length))

    def __filter_directly(self, if_function):
        """Apply if_function to the table, generator version."""

        for iatm, jatm_beg, jatm_end in self.__base_generater:

            jatm_beg_new = 0
            jatm_end_new = jatm_beg
            
            for jatm in range(jatm_beg, jatm_end+1):

                if if_function(iatm, jatm):
                    if jatm_beg_new==0: jatm_beg_new = jatm
                    jatm_end_new = jatm
                else:
                    if jatm_beg_new==0: continue
                    yield iatm, jatm_beg_new, jatm_end_new
                    jatm_beg_new = 0

            else:
                if jatm_beg_new > 0:
                    yield iatm, jatm_beg_new, jatm_end_new

    def _if_cutoff(self, crd, length):
        length2 = length**2

        def wrapper(iatm, jatm):
            dist2 = numpy.dot(crd[iatm,:] - crd[jatm,:])
            return dist2 < length2

        return wrapper

    def gen_each_atom(self):
        """Generator: InteractionTable data divided by each atoms."""
        cur_iatm  = 1
        int_table = []

        for iatm, jatm_beg, jatm_end in self:

            if iatm == cur_iatm:
                int_table.append((iatm, jatm_beg, jatm_end))
            else:
                yield int_table
                cur_iatm += 1

                while cur_iatm < iatm:
                    yield []
                    cur_iatm += 1

                int_table = [(iatm, jatm_beg, jatm_end)]

        else:
            yield int_table

    def get_each_atom(self):
        if self.__cache is None:
            return list(self.gen_each_atom())
        else:
            return self.__cache


################################################################################
class InteractionTableList:

    """

    data structure:
        [jint, (iatm, jatm_begin, jatm_end)]

    >>> table1 = InteractionTable(natom)
    >>> table1
    <InteractionTable>

    exclude the bonded pair
    >>> table2 = table1.filter(if_excluded_bond)
    <InteractionTable>

    >>> list(table2)

    perform cutoff with cutoff length to make a interaction table within
    length from each atoms
    >>> table3 = table2.cutoff(crd, 10.0)
    >>> table.map()
    """

    _memory_limit = 10 # MB

    def __init__(self, natom=None, base_table=None):
        self.__natom = natom
        if self.__natom:
            self.__base_generater = self._init_table(self.__natom)
        else:
            if base_table:
                self.__base_generater = base_table
            else:
                self.__base_generater = []

        self.__filters = []
        self.__ifilters = []
        self.__cnt = -1
        self.__num_interacts = 0
        self.__table = []

    def _init_table(self, natom):
        for iatm_1 in range(natom-1):
            iatm = iatm_1 + 1
            yield (iatm, iatm+1, natom)

    def filter_iatom(self, if_fun):
        self.__ifilters.append(if_fun)
        return self

    def filter(self, if_fun):
        self.__filters.append(if_fun)
        return self 

    def filter_or(self, iatm_beg, iatm_end):
        self.__filters.append( ('or', iatm_beg, iatm_end) )
        return self

    def filter_and(self, iatm_beg, iatm_end):
        self.__filters.append( ('and', iatm_beg, iatm_end) )
        return self

    def filter_fun(self, if_fun):
        self.__filters.append( ('fun', if_fun) )
        return self

    def insert_filter(self, if_fun):
        self.__filters.insert(0, if_fun)
        return self

    def list(self):
        return self.__apply()

    def __iter__(self):
        return self

    def next(self):
        self.__cnt += 1

        if self.__num_interacts == 0:
            self.__apply()

        if self.__cnt >= self.__num_interacts:
            raise StopIteration()

        return self.__table[self.__cnt]

    __next__ = next # version 3.x

    def copy(self):
        gen1, gen2 = itertools.tee(self.__apply())
        self.__apply = gen1
        self.__filters = []
        self.__ifilters = []
        return InteractionTable(base_table=gen2)

    def __apply(self):
        """Apply if_function to the table."""

        def gen_filter_or(table, i_beg, i_end):
            for iatm, jatm_beg, jatm_end in table:

                if iatm < i_beg:
                    yield iatm, i_beg, i_end

                elif i_beg <= iatm <= i_end:
                    yield iatm, jatm_beg, jatm_end

                elif i_end < iatm:
                    break 

                else:
                    pass

        def gen_filter_and(table, i_beg, i_end):
            for iatm, jatm_beg, jatm_end in table:

                if iatm < i_beg:
                    pass

                elif i_beg <= iatm <= i_end-1:
                    yield iatm, jatm_beg, jatm_end

                elif i_end-1 < iatm:
                    break 

                else:
                    pass

        def gen_filter_fun(table, if_fun):

            for iatm, jatm_beg, jatm_end in table:
                
                jatm_beg_new = 0
                jatm_end_new = jatm_beg
                
                for jatm in range(jatm_beg, jatm_end+1):

                    if not if_fun(iatm, jatm):
                        if jatm_beg_new > 0:
                            yield iatm, jatm_beg_new, jatm_end_new
                            jatm_beg_new = 0

                    else:
                        if jatm_beg_new==0: jatm_beg_new = jatm
                        jatm_end_new = jatm

                else:
                    if jatm_beg_new > 0:
                        yield iatm, jatm_beg_new, jatm_end_new

        table = self.__base_generater
        for f in self.__filters:
            ftype = f[0]

            if ftype == 'or':
                i_beg, i_end = f[1], f[2]
                next_table = gen_filter_or(table, i_beg, i_end)

            elif ftype == 'and':
                i_beg, i_end = f[1], f[2]
                next_table = gen_filter_and(table, i_beg, i_end)

            elif ftype == 'fun':
                if_fun = f[1]
                next_table = gen_filter_fun(table, if_fun)

            else:
                pass

            table = next_table

        self.__table = list(table)
        self.__num_interacts = len(self.__table)

    def cutoff(self, crd, length):
        return self.filter(self._if_cutoff(crd, length))

    def _if_cutoff(self, crd, length):
        length2 = length**2

        def wrapper(iatm, jatm):
            dist2 = numpy.dot(crd[iatm,:] - crd[jatm,:])
            return dist2 < length2

        return wrapper

    def get_size_mem(self, iatm_min, iatm_max, jatm_min, jatm_max):
        scale = 1
        ibound = iatm_max - iatm_min + 1
        jbound = jatm_max - jatm_min + 1
        memory = 8 * ibound * jbound * 3 * scale
        return memory

    def get_size_pair(self, npair):
        scale = 1
        memory = 8 * npair * 3 * scale
        return memory

    get_size = get_size_pair

    def gen_each_atom(self):
        """Generator: InteractionTable data divided by each atoms."""
        cur_iatm  = 1
        table = []

        for iatm, jatm_beg, jatm_end in self:

            if iatm == cur_iatm:
                table.append((iatm, jatm_beg, jatm_end))
            else:
                yield table
                cur_iatm += 1

                while cur_iatm < iatm:
                    yield []
                    cur_iatm += 1

                table = [(iatm, jatm_beg, jatm_end)]

        else:
            yield table

    def gen_divided_table_old(self, num_pair_limits=None):
        """Generate a table divided to each number of pair."""

        if num_pair_limits is None:
            num_pair_limits = self._limit_pair

        npair = 0
        table = []

        for iatm, jatm_beg, jatm_end in self:
            npair += jatm_end - jatm_beg + 1
            table.append( (iatm, jatm_beg, jatm_end) )

            if npair > num_pair_limits:
                yield table
                npair = 0
                table = []

        else:
            if table:
                yield table

    def gen_divided_table_pair(self, mem_limit=None):
        """Generate a table divided to each number of pair."""

        self.__memories = []

        if mem_limit is None:
            mem_limit = self._memory_limit

        mem_lower = mem_limit * 10**6
        mem_upper = 1.2 * mem_lower

        npair = 0
        table = []
        npair_p = 0

        base_table = list(self)

        for itab, (iatm, jatm_beg, jatm_end) in enumerate(base_table):

            table += [(iatm, jatm_beg, jatm_end)]
            npair += jatm_end - jatm_beg + 1
            mem = self.get_size(npair)

            if mem_lower <= mem <= mem_upper:
                self.__memories.append( mem )
                yield table
                npair = 0
                table = []

            elif mem_upper < mem:
                iatm_p, jatm_beg_p, jatm_end_p = table.pop()
                mem = self.get_size(npair_p)
                self.__memories.append( mem )
                yield table
                npair = jatm_beg_p - jatm_beg_p + 1
                table = [(iatm_p, jatm_beg_p, jatm_end_p)]

            npair_p = npair

        else:
            mem = self.get_size(npair_p)
            self.__memories.append( mem )
            if table: yield table

    def gen_divided_table_memory(self):
        """Generate a table divided by memory limit."""

        # if self._memory_limit is None:
            # memory_limt = self._memory_limit

        self.__memories = []
        memory_lower = self._memory_limit * 10**3
        memory_upper = 1.2 * memory_lower
        base_table = list(self)

        for itab, (iatm,jatm_beg,jatm_end) in enumerate(base_table):

            if itab==0:
                iatm_min, iatm_max = iatm, iatm
                jatm_min, jatm_max = jatm_beg, jatm_end
                table = [(iatm, jatm_beg, jatm_end)]
                memory = 0
                continue

            table += [(iatm, jatm_beg, jatm_end)]
            iatm_max = iatm
            jatm_min = min(jatm_beg, jatm_min)
            jatm_max = max(jatm_end, jatm_max)

            memory = self.get_size(iatm_min, iatm_max, jatm_min, jatm_max)

            if memory_lower < memory < memory_upper:
                self.__memories.append( memory )
                # print('memory', iatm_min, iatm_max, jatm_min, jatm_max)
                print('bound', iatm_max-iatm_min+1, jatm_max-jatm_min+1)
                yield table
                iatm_min = iatm
                jatm_min = jatm_end
                table = []

            elif memory_upper <= memory:
                # print('memory', iatm_min, iatm_max, jatm_min, jatm_max)
                iatm_p, jatm_beg_p, jatm_end_p  = table.pop()
                # print('bound', iatm_max-iatm_min+1, jatm_max-jatm_min+1)
                memory = self.get_size(
                        iatm_min_p, iatm_max_p, jatm_min_p, jatm_max_p)
                print('bound', iatm_max_p-iatm_min_p+1, jatm_max_p-jatm_min_p+1)
                self.__memories.append( memory )
                yield table
                iatm_min = iatm_p
                jatm_min = jatm_end_p
                table = [(iatm_p, jatm_beg_p, jatm_end_p)]

            iatm_min_p = iatm_min
            iatm_max_p = iatm_max
            jatm_min_p = jatm_min
            jatm_max_p = jatm_max
            
        else:
            memory = self.get_size(iatm_min, iatm_max, jatm_min, jatm_max)
            self.__memories.append( memory )
            if table: yield table

    gen_divided_table = gen_divided_table_pair

    def get_table_memories(self):
        return self.__memories


InteractionTable = InteractionTableList

################################################################################
def make_accesible_table(pairs, natom):
    """Convert the given pair list to easily accessible table.
    pair_table: iatm => (jatm1, jatm2, ...)
    """
    # recreate the bonded pair list to make non-bonded interaction table
    # effectively.
    pair_table = [ [] for iatm in range(natom) ]
    for iatm, jatm in pairs:
        pair_table[iatm-1].append(jatm)
    return pair_table


if __name__ == '__main__':

    target = [5,7,8,9]

    def if_fun_or(iatm, jatm):
        return (iatm in target) or (jatm in target)

    def if_fun_and(iatm, jatm):
        return (iatm in target) and (jatm in target)

    # def if_fun2(iatm, jatm):
    #     return 101 <= iatm <= 200

    # def if_fun3(iatm):
    #     return iatm in range(100, 200)


    natom = 5000
    from benchmarker import Benchmarker
    with Benchmarker(width=20) as bm:

        with bm('or'):
            print()
            table1 = InteractionTable(natom)
            t = table1.filter_or(5, 10)
            for a in t.gen_divided_table(5000):
                print(a)

        with bm('and'):
            print()
            table2 = InteractionTable(natom)
            t = table2.filter_and(10, 20)
            for a in t.gen_divided_table(40):
                print(a)
                
        # with bm('if_fun_or'):
        #     print()
        #     table3 = InteractionTable(natom)
        #     t = table3.filter_fun(if_fun_or)
        #     for a in t.gen_divided_table(40):
        #         print(a)

        # with bm('if_fun_and'):
        #     print()
        #     table4 = InteractionTable(natom)
        #     t = table4.filter_fun(if_fun_and)
        #     for a in t.gen_divided_table(40):
        #         print(a)

        with bm('or, if_fun_or'):
            print()
            table = InteractionTable(natom)
            t = table.filter_or(5, 10).filter_fun(if_fun_or)
            for a in t.gen_divided_table(40):
                print(a)

